/*
 * Copyright (c) 2009 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on 2010-01-25
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.clc;

import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;

class AddIdentityAction extends AbstractAction
{
	AddIdentityAction(ConnectionManager cm)
	{
		super(cm, "addIdentity", "Creates a new identity. Syntax:\n" +
			"  addIdentity <dn|x509|email> <value> [label]\n" +
			"or:" +
			"  addIdentity email <value> passwd <password> [label]\n" +
			"The first argument is an identity type. Next is value, " +
			"which is a simple string in case of 'dn' or 'email' types and " +
			"a file name with " +
			"certificate in case of 'x509' type. 3rd (optional) " +
			"argument is a friendly " +
			"(unique) name for the identity. It is common for all equivalent " +
			"identities. When adding email type identity it is also possible" +
			" to set it's password.\n" + 
			"Example:\n" +
			"  addIdentity email test@example.com Johnny\n", 2);
	}
	
	@Override
	public boolean invoke(String[] args) throws UVOSException
	{
		Identity i;
		if (args.length > 4 && args[3].equals("passwd"))
		{
			i = getIdentity(args[1], args[2], 
					args.length > 5 ? args[5] : null);
			i.setAuthnData(args[4]);
		} else
			i = getIdentity(args[1], args[2], 
				args.length >= 4 ? args[3] : null);
		if (i == null)
			return true;
		
		cm.voMan.addIdentity(i);
		System.out.println("Added identity: " + i);
		return true;
	}
}