/*
 * Copyright (c) 2009 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on 2010-01-25
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.clc;

import pl.edu.icm.unicore.uvos.api.Element;
import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;

class DisableAttributeAction extends AbstractAction
{
	DisableAttributeAction(ConnectionManager cm)
	{
		super(cm, "disableAttribute", "Disables an attribute. Syntax:\n" +
			"  disableAttribute <global> <dn|x509|email> <identity> " +
			"<AttributeName> [value]\n" +
			"  disableAttribute <group> <groupPath> " +
			"<AttributeName> [value]\n" +
			"  disableAttribute <ig> <dn|x509|email> <identity> <groupPath>" +
			"<AttributeName> [value]\n" + 
			"The first argument specifies what kind of attribute will be changed: " +
			"group attribute, identity global attribute, or identity attribute " +
			"valid only in particular group. If the optional 'value' argument " +
			"is set then only this value of attribute will be disabled.\n" +
			"Example:\n" +
			"  disableAttribute ig email ann@example.com /group " +
			"urn:unicore:attrType:user:proffession scientist\n", 3);
	}
	
	@Override
	public boolean invoke(String[] args) throws UVOSException
	{
		Element e = getElement(args, 1, args.length);
		if (e == null)
			return true;
		int next = 3;
		if (e.getIdentity() != null)
		{
			next++;
			if (e.getGroup() != null)
				next++;
		}
		if (next >= args.length)
		{
			System.out.println("Not enough arguments.");
			return true;
		}
		String attr = args[next];
		String value = null;
		if (next + 1 < args.length)
			value = args[next+1];
		cm.voMan.disableAttribute(e, attr, value);
		if (value == null)
			System.out.println("Disabled attribute " + 
				attr + " of element " + e);
		else
			System.out.println("Disabled value " + value + " of attribute " + 
				attr + " of element " + e);
		return true;
	}
}