/*
 * Copyright (c) 2009 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on 2010-01-25
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.clc;

import java.util.List;

import pl.edu.icm.unicore.uvos.api.Attribute;
import pl.edu.icm.unicore.uvos.api.Element;
import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;

class GetAllAttributesAction extends AbstractAction
{
	GetAllAttributesAction(ConnectionManager cm)
	{
		super(cm, "getAttributes", "Returns all attributes of the given element.\n" +
			"Syntax:\n" +
			"   getAttributes <global> <dn|x509|email> <identity> " +
			"[allScopes] [includeImpliedGroups]\n" +
			"   getAttributes <group> <groupPath>\n" +
			"   getAttributes <ig> <dn|x509|email> <identity> " +
			"<groupPath>\n" +
			"The first argument specifies what kind of attribute will be listed: " +
			"group attribute, identity global attribute, or identity attribute " +
			"valid only in particular group.\n" +
			"Example:\n" +
			"  getAttributes ig email ann@example.com /group\n", 2);
	}
	
	@Override
	public boolean invoke(String[] args) throws UVOSException
	{
		Element e = getElement(args, 1, args.length);
		if (e == null)
			return true;
		int next = 3;
		if (e.getIdentity() != null)
		{
			next++;
			if (e.getGroup() != null)
				next++;
		}
		boolean scopes = false;
		boolean implied = false;
		if (next < args.length)
			scopes = Boolean.parseBoolean(args[next++]);
		if (next < args.length)
			implied = Boolean.parseBoolean(args[next++]);

		System.out.println("Effective attributes are:\n");
		List<Attribute> list;
		if (cm.qTime == null)
			list = cm.voQuery.getAttributes(e, null, true, 
				scopes, implied);
		else
			list = cm.voQueryHistory.getAttributes(e, null, true, 
					scopes, implied, cm.qTime);
		for (Attribute a: list)
			System.out.println("  " + a);
		System.out.println("Exact attributes are:\n");
		List<Attribute> list2;
		if (cm.qTime == null)
			list2 = cm.voQuery.getAttributes(e, null, false, 
				scopes, implied);
		else
			list2 = cm.voQueryHistory.getAttributes(e, null, false, 
					scopes, implied, cm.qTime);
		for (Attribute a: list2)
			System.out.println("  " + a);
		return true;
	}
}