/*
 * Copyright (c) 2009 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on 2010-01-25
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.clc;

import java.util.List;

import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;

class GetAllEquivalentsAction extends AbstractAction
{
	GetAllEquivalentsAction(ConnectionManager cm)
	{
		super(cm, "getAllEquivalents", "Retrieves a list of all identities that are equivalent " +
			"to the given one.\n" +
			"Syntax:\n" +
			"   getAllEquivalents <dn|x509|email> <identity1>\n" + 
			"Example:\n" +
			"   getAllEquivalents email ann@example.com\n", 2);
	}
	
	@Override
	public boolean invoke(String[] args) throws UVOSException
	{
		Identity i = getIdentity(args[1], args[2], null);
		if (i == null)
			return true;
		List<Identity> equivs;
		if (cm.qTime == null)
			equivs = cm.voQuery.getAllEquivalents(i);
		else
			equivs = cm.voQueryHistory.getAllEquivalents(i, cm.qTime);
		
		System.out.println("The following identities are equivalent:");
		for (Identity id: equivs)
			System.out.println("  " + id);
		return true;
	}
}