/*
 * Copyright (c) 2009 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on 2010-01-25
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.clc;

import java.util.List;

import pl.edu.icm.unicore.uvos.api.Group;
import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;

class GetAllGroupsAction extends AbstractAction
{
	GetAllGroupsAction(ConnectionManager cm)
	{
		super(cm, "getAllGroups", "Retrieves a list of all groups the given " +
			"identity is a member of.\n" +
			"Syntax:\n" +
			"   getAllGroups <dn|x509|email> <identity> [implied]\n" +
			"If the last arg is given then also groups implied are " +
			"returned, i.e. if user is a member of group /A/B then /A" +
			"will be returned then too." + 
			"Example:\n" +
			"   getAllGroups email ann@example.com\n", 2);
	}
	
	@Override
	public boolean invoke(String[] args) throws UVOSException
	{
		Identity i = getIdentity(args[1], args[2], null);
		if (i == null)
			return true;
		boolean implied = false;
		if (args.length > 3 && args[3].equals("implied"))
			implied = true;
		List<Group> groups;
		if (cm.qTime == null)
			groups = cm.voQuery.getAllGroups(i, implied);
		else
			groups = cm.voQueryHistory.getAllGroups(i, implied, cm.qTime);
			
		System.out.println(i + " is a member of the following groups:");
		for (Group g: groups)
			System.out.println("  " + g);
		return true;
	}
}