/*
 * Copyright (c) 2009 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on 2010-01-25
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.clc;

import java.util.List;

import pl.edu.icm.unicore.uvos.api.Attribute;
import pl.edu.icm.unicore.uvos.api.Element;
import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;

class GetAttributeAction extends AbstractAction
{
	GetAttributeAction(ConnectionManager cm)
	{
		super(cm, "getAttribute", "Returns value(s) of the attribute of the given element.\n" +
			"Syntax:\n" +
			"   getAttribute <global> <dn|x509|email> <identity> <attribute>" +
			" [allScopes] [includeImpliedGroups]\n" +
			"   getAttribute <group> <groupPath> <attribute>\n" +
			"   getAttribute <ig> <dn|x509|email> <identity> <groupPath>" +
			" <attribute>\n" + 
			"The first argument specifies what kind of attribute will be listed: " +
			"group attribute, identity global attribute, or identity attribute " +
			"valid only in particular group.\n" +
			"Example:\n" +
			"  getAttribute ig email ann@example.com /group " +
			"urn:unicore:attrType:user:proffession\n", 3);
	}
	
	@Override
	public boolean invoke(String[] args) throws UVOSException
	{
		Element e = getElement(args, 1, args.length);
		if (e == null)
			return true;
		int next = 3;
		if (e.getIdentity() != null)
		{
			next++;
			if (e.getGroup() != null)
				next++;
		}
		if (next >= args.length)
		{
			System.out.println("Not enough arguments.");
			return true;
		}
		String attr = args[next];
		boolean scopes = false;
		boolean implied = false;
		if (next < args.length)
			scopes = Boolean.parseBoolean(args[next++]);
		if (next < args.length)
			implied = Boolean.parseBoolean(args[next++]);
		
		System.out.println("Effective attributes are: ");
		
		List<Attribute> ret;
		if (cm.qTime == null)
			ret = cm.voQuery.getAttributes(e, attr, true, scopes, implied);
		else
			ret = cm.voQueryHistory.getAttributes(e, attr, true, 
					scopes, implied, cm.qTime);
		for (Attribute a: ret)
			System.out.println(a.getName());
		System.out.println("Exact attributes are: ");
		List<Attribute> ret2;
		if (cm.qTime == null)
			ret2 = cm.voQuery.getAttributes(e, attr, false, scopes, implied);
		else
			ret2 = cm.voQueryHistory.getAttributes(e, attr, false, 
					scopes, implied, cm.qTime);
		for (Attribute a: ret2)
			System.out.println(a);
		return true;
	}
}