/*
 * Copyright (c) 2011 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on 03-09-2011
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */
package pl.edu.icm.unicore.uvos.clc;

import org.apache.xmlbeans.XmlOptions;

import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;

/**
 * Retrieves information from the UVOS GLUE 2 endpoint.
 * @author K. Benedyczak
 */
public class GetGlueInfoAction extends AbstractAction
{
	public GetGlueInfoAction(ConnectionManager cm)
	{
		super(cm, "getServerInfo", "Retrieves GLUE information exposed by the server.\n" +
				"Syntax:\n" +
				"   getServerInfo [full]\n" +
				"If the optional arg is given then raw Glue 2 XML is printed. Otherwise only a parsed " +
				"information (including server's version) is presented." + 
				"Example:\n" +
				"   getServerInfo full\n", 0);
	}
	
	@Override
	public boolean invoke(String[] args) throws UVOSException
	{
		boolean full = false;
		if (args.length > 1 && args[1].equals("full"))
			full = true;
		if (full)
		{
			System.out.println(cm.glueIP.getServiceInfo().xmlText(
					new XmlOptions().setSavePrettyPrint()));
		} else
		{
			System.out.println(cm.glueIP.getServerDescription());
		}
		return true;
	}
}
