/*
 * Copyright (c) 2009 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on 2010-01-25
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.clc;

import java.util.List;

import pl.edu.icm.unicore.uvos.api.Attribute;
import pl.edu.icm.unicore.uvos.api.Element;
import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;

class ListDisabledAttributesAction extends AbstractAction
{
	ListDisabledAttributesAction(ConnectionManager cm)
	{
		super(cm, "getDisabledAttributes", "Lists all disabled attributes. Only disabled values are presented. " +
			"Syntax:\n" +
			"   getDisabledAttributes <global> <dn|x509|email> <identity>\n" +
			"   getDisabledAttributes <group> <groupPath>\n" +
			"   getDisabledAttributes <ig> <dn|x509|email> <identity> " +
			"<groupPath>\n" + 
			"The first argument specifies what kind of attribute will be listed: " +
			"group attribute, identity global attribute, or identity attribute " +
			"valid only in particular group.\n" +
			"Example:\n" +
			"  getDisabledAttributes ig email ann@example.com /group\n", 
			2, "listDisabledAttributes");
	}
	
	@Override
	public boolean invoke(String[] args) throws UVOSException
	{
		Element e = getElement(args, 1, args.length);
		if (e == null)
			return true;
		List<Attribute> list = cm.voMan.getDisabledAttributes(e);
		System.out.println("Disabled attributes are:\n");
		for (Attribute a: list)
			System.out.println("  " + a);
		return true;
	}
}