/*
 * Copyright (c) 2009 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on 2010-01-25
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.clc;

import pl.edu.icm.unicore.uvos.api.Attribute;
import pl.edu.icm.unicore.uvos.api.Group;
import pl.edu.icm.unicore.uvos.api.PermissionDesignator;
import pl.edu.icm.unicore.uvos.api.PermissionDesignator.SpecialDesignator;
import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;

class RemoveAuthZAction extends AbstractAction
{
	RemoveAuthZAction(ConnectionManager cm)
	{
		super(cm, "removeAuthz", 
			"Removes authorization policy for VO service access\n" +
			"Syntax:\n" +
			"   removeAuthz <group> a <AttributeName>\n" +
			"   removeAuthz <group> <o|m>\n" +
			"Use group '/' to modify global policy. Permissions can removed either " +
			"from the bearer of an attribute or from the accessed resource owner ('o' case) " +
			"or from the group members ('m' case')\n" +
			"Example:\n" +
			"  removeAuthz /some/group m", 2);
	}
	
	@Override
	public boolean invoke(String[] args) throws UVOSException
	{
		Group g = null;
		if (!args[1].equals("/"))
			g = new Group(args[1]);
		PermissionDesignator designator;
		if (args[2].equals("a"))
		{
			Attribute a = getAttribute(args[3], args, 4, 4);
			designator = new PermissionDesignator(a);
		} else if (args[2].equals("o"))
		{
			designator = new PermissionDesignator(
					SpecialDesignator.OWNER);
		} else if (args[2].equals("m"))
		{
			designator = new PermissionDesignator(
					SpecialDesignator.MEMBER);				
		} else
		{
			System.out.println("Wrong type specified " +
					"(must be 'a', " +
					"'m' or 'o')");
			return true;
		}
		cm.voAuthz.modifyPermissions(g, designator, null);
		return true;
	}
}