/*
 * Copyright (c) 2009 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on 2010-01-25
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.clc;

import pl.edu.icm.unicore.uvos.api.Group;
import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;

class RemoveFromGroupAction extends AbstractAction
{
	RemoveFromGroupAction(ConnectionManager cm)
	{
		super(cm, "removeFromGroup", 
			"Removes an identity from a group. Syntax:\n" +
			"  removeFromGroup <dn|x509|email> <identity> <groupPath>\n" +
			"The first argument is an identity type. Next is value, " +
			"which is a simple string in case of 'dn' or 'email' types and a " +
			"file name with " +
			"X509 certificate in case of 'x509' type. It is the value of " +
			"added identity. " +
			"The last argument is the group path.\n" + 
			"Example:\n" +
			"  removeFromGroup email john@example.com /group/subgroup\n", 3);
	}
	
	@Override
	public boolean invoke(String[] args) throws UVOSException
	{
		Identity id = getIdentity(args[1], args[2], 
				args.length == 3 ? args[3] : null);
		if (id == null)
			return true;
		Group g = new Group(args[3]);
		cm.voMan.removeFromGroup(id, g);
		System.out.println("Removed identity " + id + " from the group: " + g);
		return true;
	}
}