/*
 * Copyright (c) 2009 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on 2010-01-25
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.clc;

import java.io.File;

import pl.edu.icm.unicore.uvos.api.VOApplicationForm;
import pl.edu.icm.unicore.uvos.api.exceptions.InternalException;
import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.XMLBeansMapper;
import xmlbeans.pl.edu.icm.uvos.internal.app.VOApplicationFormDocument;

class SetApplicationFormAction extends AbstractAction
{
	SetApplicationFormAction(ConnectionManager cm)
	{
		super(cm, "setApplicationForm", 
			"Adds or updates application form. Definition is " +
			"read form XML file\n" +
			"Syntax:\n" +
			"   setApplicationForm <update:true|false> " +
			"	<fileWithDefinition>\n" +
			"Example:\n" +
			"   setApplicationForm true appForm3.xml", 2);
	}
	
	@Override
	public boolean invoke(String[] args) throws UVOSException
	{
		boolean update = false;
		if (args[1].equals("true"))
			update = true;
		VOApplicationFormDocument formDoc;
		try
		{
			formDoc = VOApplicationFormDocument.Factory.parse(new File(args[2]));
		} catch (Exception e)
		{
			throw new InternalException("Can't parse the file " + 
					args[2] + ": " + e);
		}
		VOApplicationForm appDef = 
			XMLBeansMapper.map2APIVOApplicationForm(formDoc);
		cm.voApps.updateApplicationForm(appDef, update);
		if (!update)
			System.out.println("Added a new application form with name "
					+ appDef.getFriendlyName());
		else
			System.out.println("Updated application form with id " 
					+ appDef.getId());
		return true;
	}
}