/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Aug 9, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.client.util;

import javax.net.ssl.SSLContext;

import org.apache.log4j.Logger;

import eu.unicore.security.util.client.IAuthenticationConfiguration;

import pl.edu.icm.unicore.uvos.util.Configuration;

/**
 * @author K. Benedyczak
 */
public class ConfigHelper implements IAuthenticationConfiguration
{
	protected Configuration conf;
	private static final Logger log = Logger.getLogger(ConfigHelper.class);
	
	public ConfigHelper(Configuration conf)
	{
		this.conf = conf;
	}
	
	public int getServerPort()
	{
		return conf.getAsInt("port", 2020, log);
	}

	public String getServerHost()
	{
		return conf.getAsString("host", "localhost", log);
	}

	public String getProtocol()
	{
		return conf.getAsString("protocol", "http", log);
	}
	
	public String getServicePath()
	{
		return conf.getAsString("servicePath", "", log);
	}
	
	public boolean doHttpAuthn()
	{
		return conf.getAsBool("HTTPauthenticate", false, log);
	}

	public String getHttpUser()
	{
		return conf.getAsString("HTTPuser", "test", log);
	}

	public String getHttpPassword()
	{
		return conf.getAsString("HTTPpasswd", "test", log);
	}
	
	public boolean doSSLAuthn()
	{
		return conf.getAsBool("SSLauthenticate", true, log);
	}	
	
	public String getKeystore()
	{
		return conf.getAsString("keystore", "conf/certs/dummyClient.jks", log);
	}

	public String getKeystorePassword()
	{
		return conf.getAsString("keystorePasswd", "the!client", log);
	}

	public String getKeystoreAlias()
	{
		return conf.getAsString("keystoreAlias", "mykey", log);
	}

	public String getKeystoreKeyPassword()
	{
		if (conf.getProperty("keystoreKeyPasswd") == null)
			return getKeystorePassword();
		return conf.getAsString("keystoreKeyPasswd", "the!client", log);
	}

	public String getKeystoreType()
	{
		return conf.getAsString("keystoreType", "JKS", log);
	}

	public String getTruststore()
	{
		return conf.getAsString("truststore",
				"conf/certs/dummyClientTruststore.jks", log);
	}

	public String getTruststorePassword()
	{
		return conf.getAsString("truststorePasswd", "the!client", log);
	}

	public String getTruststoreType()
	{
		return conf.getAsString("truststoreType", "JKS", log);
	}

	public IAuthenticationConfiguration clone()
	{
		return new ConfigHelper(conf);
	}

	public SSLContext getSSLContext()
	{
		return null;
	}
	
	public Configuration getConfiguration()
	{
		return conf;
	}
}
