/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Aug 9, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.wsclient;

import java.io.FileInputStream;
import java.security.KeyStore;
import java.security.cert.X509Certificate;
import java.util.List;

import javax.security.auth.x500.X500Principal;

import org.apache.xmlbeans.XmlOptions;
import org.w3c.dom.Document;

import eu.unicore.security.dsig.DigSignatureUtil;

import pl.edu.icm.unicore.uvos.api.Attribute;
import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.IdentityType;
import pl.edu.icm.unicore.uvos.client.util.ConfigHelper;
import pl.edu.icm.unicore.uvos.client.util.ConfigLoader;
import pl.edu.icm.unicore.uvos.util.Configuration;
import pl.edu.icm.unicore.uvos.wsclient.samlapi.SAMLVOQueryClient;

/**
 * Uses SAML query interface to get self attributes.
 * @author K. Benedyczak
 */
public class SAMLSelfInfoClient
{

	/**
	 * @param args
	 */
	public static void main(String[] args)
	{
		ConfigHelper conf = null;
		try
		{
			String cf = null;
			if (args.length > 1 && args[0].equals("-c"))
				cf = args[1];
			Configuration c = ConfigLoader.readConfig(cf);
			conf = new ConfigHelper(c);
		} catch (Exception e1)
		{
			e1.printStackTrace();
			System.exit(1);
		}
		
		String addr = conf.getProtocol() + "://" + 
			conf.getServerHost() + ":" + conf.getServerPort() + 
			conf.getServicePath();
		System.out.println("Using address: " + addr);
		runHLATest(conf, addr);
	
	}

	static void runHLATest(ConfigHelper conf, String addr) 
	{
		
		SAMLVOQueryClient client = null;
		try
		{
			String dn = getDN(conf);
			Identity subject = new Identity(IdentityType.DN, dn);
			System.out.println("Quering for DN: " + dn);
			client = new SAMLVOQueryClient(addr, conf, conf.getConfiguration());
			client.setDSigPolicy(false, true, null);
			
			List<Attribute> attrs; 
			
			attrs = client.getAttributes(subject);
			for (Attribute a: attrs)
				System.out.println("Got attr: " + a);
			Document doc = (Document) client.getLastResponse().getDomNode();
			String dump = DigSignatureUtil.dumpDOMToString(doc);
			System.out.println("\n\nRESPONSE WAS (via DOM):\n " + 
					dump + "\n\n");

			System.out.println("\n\nRESPONSE WAS:\n " + 
					client.getLastResponse().xmlText(
					new XmlOptions().setSavePrettyPrint()) + "\n\n");
		} catch (Exception e)
		{
			e.printStackTrace();
			if (client != null && client.getLastResponse() != null)
				System.out.println("\nRESPONSE WAS: " + 
					client.getLastResponse().xmlText(
					new XmlOptions().setSavePrettyPrint()));
		}
	}
	
	static String getDN(ConfigHelper conf) throws Exception
	{
		KeyStore ks = KeyStore.getInstance(conf.getKeystoreType());
		ks.load(new FileInputStream(conf.getKeystore()), 
				conf.getKeystorePassword().toCharArray());
		X509Certificate cert = (X509Certificate) 
			ks.getCertificate(conf.getKeystoreAlias());
		return cert.getSubjectX500Principal().getName(X500Principal.RFC2253);
	}
}









