/*
 * Copyright (c) 2011 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on 03-09-2011
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */
package pl.edu.icm.unicore.uvos.wsclient.api;

import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;
import xmlbeans.pl.edu.icm.uvos.cisprovider.GetServiceInfoResponseDocument;
import xmlbeans.pl.edu.icm.uvos.glue2.ServiceT;

/**
 * Client interface for querying the UVOS GLUE endpoint.
 * 
 * @author K. Benedyczak
 */
public interface UVOSGLUEExtInterface
{
	/**
	 * @return a string with a server's version. 
	 * @throws UVOSException
	 */
	public String getServerVersion() throws UVOSException;
	
	/**
	 * @return a multiline string which contains information on server's distribution,
	 * or empty string if distribution information is unknown.
	 * @throws UVOSException
	 */
	public String getServerDistribution() throws UVOSException;
	
	/**
	 * @return a multiline string with basic information about the server including its
	 * name, version and distribution.
	 * @throws UVOSException
	 */
	public String getServerDescription() throws UVOSException;
	
	/**
	 * @return an array with GLUE 2 description of the services as retrieved from the server. 
	 * @throws UVOSException
	 */
	public ServiceT[] getServicesInformation() throws UVOSException;
	
	/**
	 * @return a raw GLUE 2 document as retrieved from the server. 
	 * @throws UVOSException
	 */
	public GetServiceInfoResponseDocument getServiceInfo() throws UVOSException;
}
