/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Mar 14, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.wsclient.api;

import java.util.List;

import pl.edu.icm.unicore.uvos.api.ApplicationActions;
import pl.edu.icm.unicore.uvos.api.ApplicationStatus;
import pl.edu.icm.unicore.uvos.api.VOApplication;
import pl.edu.icm.unicore.uvos.api.VOApplicationForm;
import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;

/**
 * This interface encapsulates all engine operations which 
 * regard defining and submitting applications to VOs.
 * 
 * @author K. Benedyczak
 */
public interface VOApplicationExtInterface
{	
	/**
	 * Adds or updates an application definition. 
	 * <p>
	 * Requires a write perm for the group which is set in applicationDef.
	 * 
	 * @param applicationDef
	 * @param update
	 * @throws UVOSException
	 */
	public void updateApplicationForm(VOApplicationForm applicationDef,
			boolean update)
		throws UVOSException;

	/**
	 * Removes an application definition. 
	 * <p>
	 * Requires a write perm for the group which is set in applicationDef.
	 * @param id
	 * @throws UVOSException
	 */
	public void removeApplicationForm(int id)
		throws UVOSException;

	/**
	 * Lists all application definitions.
	 * <p>
	 * Requires global no perm.
	 * 
	 * @return
	 * @throws UVOSException
	 */
	public List<VOApplicationForm> getApplicationForms()
		throws UVOSException;
	
	/**
	 * Lists all applications of the selected form and/or with selected status.
	 * Both filtering arguments may be null, what eleminates the constraint.
	 * <p>
	 * Requires fullRead perm for the group, which is the application form's base. 
	 * In case of getting applications of any form a global fullRead is required.  
	 * @param formId
	 * @return
	 * @throws UVOSException
	 */
	public List<VOApplication> getApplications(Integer formId, ApplicationStatus status)
		throws UVOSException;
	
	/**
	 * Adds a new application.
	 * <p>
	 * Requires no perm.
	 * @param application
	 * @throws UVOSException
	 */
	public void submitApplication(VOApplication application)
		throws UVOSException;
	
	/**
	 * Process an application. This operation only marks the application 
	 * accordingly but doesn't add a new identity (it must be performed by client
	 * software manually).
	 * <p>
	 * Requires write perm for the group that is base of application form 
	 * which the selected application is for.
	 * @param id
	 * @param action
	 * @param notes
	 * @param application
	 * @throws UVOSException
	 */
	public void processApplication(int id, ApplicationActions action, 
			String notes, boolean sendConfirmation, VOApplication application)
		throws UVOSException;
	
	public void csrProcessedNotification(String csr, boolean accepted, 
			String certificate, boolean sendNotification)
		throws UVOSException;
}
