/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Sep 13, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.wsclient.api;

import java.util.List;

import org.apache.xmlbeans.XmlException;

import pl.edu.icm.unicore.uvos.api.ApplicationActions;
import pl.edu.icm.unicore.uvos.api.ApplicationStatus;
import pl.edu.icm.unicore.uvos.api.VOApplication;
import pl.edu.icm.unicore.uvos.api.VOApplicationForm;
import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;
import pl.edu.icm.unicore.uvos.api.exceptions.VOApplicationException;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.UVOSFault;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.WSApplicationsInterface;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.XMLBeansMapper;
import xmlbeans.pl.edu.icm.uvos.internal.app.ProcessActionType;
import xmlbeans.pl.edu.icm.uvos.internal.app.ProcessVOApplicationDocument;
import xmlbeans.pl.edu.icm.uvos.internal.app.ProcessVOApplicationType;
import xmlbeans.pl.edu.icm.uvos.internal.app.VOApplicationListDocument;

/**
 * @author K. Benedyczak
 */
public class WSApplicationClient implements VOApplicationExtInterface
{
	private WSApplicationsInterface proxy;
	
	public WSApplicationClient(WSApplicationsInterface proxy)
	{
		this.proxy = proxy;
	}

	public List<VOApplicationForm> getApplicationForms() throws UVOSException
	{
		try
		{
			return XMLBeansMapper.map2APIVOApplicationFormList(
					proxy.getApplicationForms());
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
	}

	public void removeApplicationForm(int id) throws UVOSException
	{
		try
		{
			proxy.removeApplicationForm(id);
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
	}

	public void updateApplicationForm(VOApplicationForm applicationDef, boolean update)
		throws UVOSException
	{
		try
		{
			proxy.updateApplicationForm(
				XMLBeansMapper.map2XMLVOApplicationFormDoc(applicationDef), 
				update);
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		} catch (XmlException e)
		{
			throw new VOApplicationException("Can't parse application" +
					"form constraints: " + e);
		}
	}

	public List<VOApplication> getApplications(Integer formId, ApplicationStatus status) 
		throws UVOSException
	{

		try
		{
			VOApplicationListDocument list = proxy.getApplications(
					formId, (status == null) ? null : status.toString());
			return XMLBeansMapper.map2APIVOApplicationList(list);
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
	}

	public void submitApplication(VOApplication application) throws UVOSException
	{
		try
		{
			proxy.submitApplication(
				XMLBeansMapper.map2XMLVOApplicationDoc(application));
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		} catch (XmlException e)
		{
			throw new VOApplicationException("Can't parse application's " +
					"requirements: " + e);
		}
	}

	public void processApplication(int id, ApplicationActions action, String notes, 
			boolean sendConfirmation, VOApplication application) throws UVOSException
	{
		try
		{
			ProcessVOApplicationDocument reqDoc = ProcessVOApplicationDocument.
				Factory.newInstance();
			ProcessVOApplicationType req = reqDoc.addNewProcessVOApplication();
			req.setAction(ProcessActionType.Enum.forString(action.toString()));
			req.setApplicationId(id);
			if (notes != null)
				req.setNotes(notes);
			req.setSendConfirmation(sendConfirmation);
			if (application != null)
				req.setUpdatedApplication(XMLBeansMapper.
						map2XMLVOApplication(application));
			proxy.processApplication(reqDoc);
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		} catch (XmlException e)
		{
			throw new VOApplicationException("Can't parse application's " +
					"requirements: " + e);
		}
		
	}

	public void csrProcessedNotification(String csr, boolean accepted, 
			String certificate, boolean sendNotification) 
		throws UVOSException
	{
		try
		{
			proxy.csrProcessedNotification(csr, accepted, 
					certificate, sendNotification);
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
		
	}
}
