/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Sep 13, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.wsclient.api;

import java.util.ArrayList;
import java.util.List;

import org.apache.xmlbeans.XmlException;

import pl.edu.icm.unicore.uvos.api.AttributePermission;
import pl.edu.icm.unicore.uvos.api.Group;
import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.PermissionDesignator;
import pl.edu.icm.unicore.uvos.api.Permissions;
import pl.edu.icm.unicore.uvos.api.exceptions.InvalidValueException;
import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.UVOSFault;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.WSAuthZInterface;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.XMLBeansMapper;
import xmlbeans.pl.edu.icm.uvos.internal.AttributePermissionXType;
import xmlbeans.pl.edu.icm.uvos.internal.IdentityXType;

/**
 * @author K. Benedyczak
 */
public class WSAuthZClient implements UVOSAuthZExtInterface
{
	private WSAuthZInterface proxy;
	
	public WSAuthZClient(WSAuthZInterface proxy)
	{
		this.proxy = proxy;
	}

	public Permissions checkPermissions(Group group, Identity whose) 
		throws UVOSException
	{
		try
		{
			return XMLBeansMapper.map2APIPermissions(proxy.checkPermissions(
					XMLBeansMapper.map2XMLGroupDoc(group), 
					XMLBeansMapper.map2XMLIdentityDoc(whose)).getPermissions());
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
	}

	public List<AttributePermission> getGroupAuthZ(Group group, boolean effective) 
		throws UVOSException
	{
		AttributePermissionXType[] raw;
		try
		{
			raw = proxy.getGroupAuthZ(
					XMLBeansMapper.map2XMLGroupDoc(group), effective).
					getAttributePermissionsList().getElementArray();
		} catch (UVOSFault e1)
		{
			throw e1.getAsAPIException();
		}
		ArrayList<AttributePermission> ret = new ArrayList<AttributePermission>();
		for (AttributePermissionXType apt: raw)
			try
			{
				ret.add(XMLBeansMapper.map2APIAttrPerm(apt));
			} catch (XmlException e)
			{
				throw new InvalidValueException(e.getMessage());
			}
		return ret;
	}

	public void modifyAuthenticationData(Identity id, Object newToken) 
		throws UVOSException
	{
		try
		{
			proxy.modifyAuthenticationData(XMLBeansMapper.map2XMLIdentityDoc(id), 
					newToken!= null ? newToken.toString() : null);
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
	}

	public void modifyPermissions(Group group, PermissionDesignator designator, 
			Permissions permissions) throws UVOSException
	{
		try
		{
			proxy.modifyPermissions(XMLBeansMapper.map2XMLGroupDoc(group), 
					XMLBeansMapper.map2XMLDesignatorDoc(designator), 
					XMLBeansMapper.map2XMLPermissionsDoc(permissions));
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
	}

	public List<Identity> getMyIdentities() throws UVOSException
	{
		IdentityXType raw[];
		try
		{
			raw = proxy.getMyIdentities().getIdentitiesList().getElementArray();
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
		ArrayList<Identity> ret = new ArrayList<Identity>();
		for (IdentityXType id: raw)
			ret.add(XMLBeansMapper.map2APIIdentity(id));
		return ret;
	}

	
}
