/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Sep 13, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.wsclient.api;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.xmlbeans.XmlException;

import pl.edu.icm.unicore.uvos.api.Attribute;
import pl.edu.icm.unicore.uvos.api.AttributeType;
import pl.edu.icm.unicore.uvos.api.Element;
import pl.edu.icm.unicore.uvos.api.Group;
import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.IdentityType;
import pl.edu.icm.unicore.uvos.api.Notification;
import pl.edu.icm.unicore.uvos.api.exceptions.InvalidValueException;
import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.UVOSFault;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.WSManagementInterface;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.XMLBeansMapper;
import xmlbeans.pl.edu.icm.uvos.internal.AttributeXType;

/**
 * @author K. Benedyczak
 */
public class WSManagementClient implements UVOSManagementExtInterface
{
	private WSManagementInterface proxy;
	
	public WSManagementClient(WSManagementInterface proxy)
	{
		this.proxy = proxy;
	}
	
	public Group addGroup(Group parent, String name) throws UVOSException
	{
		try
		{
			return XMLBeansMapper.map2APIGroup(proxy.addGroup(
					XMLBeansMapper.map2XMLGroupDoc(parent), name));
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
	}

	public void addIdentity(Identity toAdd) throws UVOSException
	{
		try
		{
			proxy.addIdentity(XMLBeansMapper.map2XMLIdentityDoc(toAdd));
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
	}

	public void addIdentity(Identity toAdd, Identity equivalentIdentity) 
		throws UVOSException
	{
		try
		{
			proxy.addIdentity(XMLBeansMapper.map2XMLIdentityDoc(toAdd), 
					XMLBeansMapper.map2XMLIdentityDoc(equivalentIdentity));
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
	}

	public void addToGroup(Identity toAdd, Group group) throws UVOSException
	{
		try
		{
			proxy.addToGroup(XMLBeansMapper.map2XMLIdentityDoc(toAdd), 
					XMLBeansMapper.map2XMLGroupDoc(group));
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
	}

	public void disableAttribute(Element whose, String toDisable, String valueToDisable)
		throws UVOSException
	{
		try
		{
			proxy.disableAttribute(XMLBeansMapper.map2XMLElementDoc(whose), 
					toDisable.toString(), valueToDisable);
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
	}

	public void enableAttribute(Element whose, String toEnable, String valueToEnable) 
		throws UVOSException
	{
		try
		{
			proxy.enableAttribute(XMLBeansMapper.map2XMLElementDoc(whose), 
					toEnable.toString(), valueToEnable);
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
	}

	public List<AttributeType> getAttributeTypes() throws UVOSException
	{
		AttributeXType[] raw;
		try
		{
			raw = proxy.getAttributeTypes().getAttributesList().getElementArray();
		} catch (UVOSFault e1)
		{
			throw e1.getAsAPIException();
		}
		ArrayList<AttributeType> ret = new ArrayList<AttributeType>();
		for (AttributeXType u: raw)
			ret.add(new AttributeType(u.getName(), u.getShortDescription(), 
					u.getDescription()));
		return ret;
	}

	public List<Attribute> getDisabledAttributes(Element whose) throws UVOSException
	{
		AttributeXType[] raw;
		try
		{
			raw = proxy.getDisabledAttributes(
					XMLBeansMapper.map2XMLElementDoc(whose)).getAttributesList().
					getElementArray();
		} catch (UVOSFault e1)
		{
			throw e1.getAsAPIException();
		}
		ArrayList<Attribute> ret = new ArrayList<Attribute>();
		for (AttributeXType u: raw)
			try
			{
				ret.add(XMLBeansMapper.map2APIAttribute(u));
			} catch (XmlException e)
			{
				throw new InvalidValueException(e.getMessage());
			}
		return ret;
	}

	public List<IdentityType> getIdentityTypes() throws UVOSException
	{
		String[] raw;
		try
		{
			raw = proxy.getIdentityTypes().getNamesList().getElementArray();
		} catch (UVOSFault e1)
		{
			throw e1.getAsAPIException();
		}
		ArrayList<IdentityType> ret = new ArrayList<IdentityType>();
		for (String u: raw)
			ret.add(new IdentityType(u));
		return ret;
	}

	public Group copyGroup(Group toCopy, Group newParent, String newName, 
			boolean removeOriginal) throws UVOSException
	{
		try
		{
			return XMLBeansMapper.map2APIGroup(proxy.copyGroup(
				XMLBeansMapper.map2XMLGroupDoc(toCopy), 
				XMLBeansMapper.map2XMLGroupDoc(newParent),
				newName, removeOriginal));
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
	}

	public void purgeHistoricalData(Date deleteFrom) throws UVOSException
	{
		try
		{
			proxy.purgeHistoricalData(XMLBeansMapper.d2c(deleteFrom));
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
	}

	public void removeAttribute(Element whom, String toRemove) throws UVOSException
	{
		try
		{
			proxy.removeAttribute(XMLBeansMapper.map2XMLElementDoc(whom), 
					toRemove.toString());
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
	}

	public void removeFromGroup(Identity toRemove, Group group) throws UVOSException
	{
		try
		{
			proxy.removeFromGroup(XMLBeansMapper.map2XMLIdentityDoc(toRemove), 
					XMLBeansMapper.map2XMLGroupDoc(group));
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
	}

	public void removeGroup(Group toRemove, boolean recursive) throws UVOSException
	{
		try
		{
			proxy.removeGroup(XMLBeansMapper.map2XMLGroupDoc(toRemove), recursive);
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
	}

	public void removeIdentity(Identity toRemove) throws UVOSException
	{
		try
		{
			proxy.removeIdentity(XMLBeansMapper.map2XMLIdentityDoc(toRemove));
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
	}

	public void setAttribute(Element whom, Attribute toAdd, boolean update) throws 
		UVOSException
	{
		try
		{
			proxy.setAttribute(XMLBeansMapper.map2XMLElementDoc(whom), 
					XMLBeansMapper.map2XMLAttributeDoc(toAdd), update);
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
	}

	public void setIdentityLabel(Identity toChange, String label) throws 
		UVOSException
	{
		try
		{
			proxy.setIdentityLabel(XMLBeansMapper.map2XMLIdentityDoc(toChange), label);
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
	}

	public void updateAttributeTypes(List<AttributeType> toUpdate, boolean clear) throws UVOSException
	{
		try
		{
			proxy.updateAttributeTypes(
				XMLBeansMapper.map2XMLAttributesListAt(toUpdate), clear);
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
	}

	public void setIdentityStatus(Identity toChange, boolean how) throws UVOSException
	{
		try
		{
			proxy.setIdentityStatus(
				XMLBeansMapper.map2XMLIdentityDoc(toChange),
				how);
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
	}

	public void addNotification(String action, String receipents, String groupFilter)
		throws UVOSException
	{
		try
		{
			proxy.addNotification(action, receipents, groupFilter);
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
	}

	public List<Notification> getNotifications(String action, String groupFilter) 
		throws UVOSException
	{
		try
		{
			return XMLBeansMapper.map2APINotificationList(
				proxy.getNotifications(action, groupFilter));
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
	}

	public void removeNotification(int id) throws UVOSException
	{
		try
		{
			proxy.removeNotification(id);
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
	}
}
