/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Sep 13, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.wsclient.api;

import java.util.ArrayList;
import java.util.List;

import org.apache.xmlbeans.XmlException;

import pl.edu.icm.unicore.uvos.api.Attribute;
import pl.edu.icm.unicore.uvos.api.Element;
import pl.edu.icm.unicore.uvos.api.Group;
import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.exceptions.InvalidValueException;
import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.UVOSFault;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.WSQueryInterface;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.XMLBeansMapper;
import xmlbeans.pl.edu.icm.uvos.internal.AttributeXType;
import xmlbeans.pl.edu.icm.uvos.internal.ElementXType;
import xmlbeans.pl.edu.icm.uvos.internal.GroupXType;
import xmlbeans.pl.edu.icm.uvos.internal.IdentityXType;

/**
 * @author K. Benedyczak
 */
public class WSQueryClient implements UVOSQueryExtInterface
{
	private WSQueryInterface proxy;
	
	public WSQueryClient(WSQueryInterface proxy)
	{
		this.proxy = proxy;
	}

	public boolean areEquivalent(Identity i1, Identity i2) throws UVOSException
	{
		try
		{
			return proxy.areEquivalent(XMLBeansMapper.map2XMLIdentityDoc(i1), 
					XMLBeansMapper.map2XMLIdentityDoc(i2));
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
	}

	public List<Attribute> getAttributes(Element owner, String attribute, boolean effective,
			boolean includeScoped, boolean includeImplied) 
		throws UVOSException
	{
		AttributeXType[] raw;
		try
		{
			String attrName = null;
			if (attribute != null)
				attrName = attribute.toString();
			raw = proxy.getAttributes(
				XMLBeansMapper.map2XMLElementDoc(owner), 
				attrName,
				effective,
				includeScoped, 
				includeImplied).
				getAttributesList().getElementArray();
		} catch (UVOSFault e1)
		{
			throw e1.getAsAPIException();
		}
		ArrayList<Attribute> ret = new ArrayList<Attribute>();
		for (AttributeXType axt: raw)
			try
			{
				ret.add(XMLBeansMapper.map2APIAttribute(axt));
			} catch (XmlException e)
			{
				throw new InvalidValueException(e.getMessage());
			}
		return ret;
	}

	public List<Identity> getAllEquivalents(Identity who) throws UVOSException
	{
		IdentityXType[] raw;
		try
		{
			raw = proxy.getAllEquivalents(
					XMLBeansMapper.map2XMLIdentityDoc(who)).
					getIdentitiesList().getElementArray();
		} catch (UVOSFault e1)
		{
			throw e1.getAsAPIException();
		}
		ArrayList<Identity> ret = new ArrayList<Identity>();
		for (IdentityXType ixt: raw)
			ret.add(XMLBeansMapper.map2APIIdentity(ixt));
		return ret;
	}

	public List<Group> getAllGroups(Identity who, boolean implied) throws UVOSException
	{
		GroupXType[] raw;
		try
		{
			raw = proxy.getAllGroups(
				XMLBeansMapper.map2XMLIdentityDoc(who),	implied).
				getGroupsList().getElementArray();
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
		ArrayList<Group> ret = new ArrayList<Group>();
		for (GroupXType ixt: raw)
			ret.add(XMLBeansMapper.map2APIGroup(ixt));
		return ret;
	}

	public List<Identity> getAllIdentities() throws UVOSException
	{
		IdentityXType[] raw;
		try
		{
			raw = proxy.getAllIdentities().getIdentitiesList().
				getElementArray();
		} catch (UVOSFault e1)
		{
			throw e1.getAsAPIException();
		}
		ArrayList<Identity> ret = new ArrayList<Identity>();
		for (IdentityXType ixt: raw)
			ret.add(XMLBeansMapper.map2APIIdentity(ixt));
		return ret;
	}

	public List<Element> getGroupContent(Group group) throws UVOSException
	{
		ElementXType[] raw;
		try
		{
			raw = proxy.getGroupContent(
					XMLBeansMapper.map2XMLGroupDoc(group)).getElementsList().
					getElementArray();
		} catch (UVOSFault e1)
		{
			throw e1.getAsAPIException();
		} 
		
		ArrayList<Element> ret = new ArrayList<Element>();
		for (ElementXType xt: raw)
			ret.add(XMLBeansMapper.map2APIElement(xt));
		return ret;
	}

	public boolean isMember(Identity who, Group group, boolean effective) 
		throws UVOSException
	{
		try
		{
			return proxy.isMember(XMLBeansMapper.map2XMLIdentityDoc(who),
					XMLBeansMapper.map2XMLGroupDoc(group), effective);
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
	}
}
