/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Sep 13, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.wsclient.api;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.apache.xmlbeans.XmlException;

import pl.edu.icm.unicore.uvos.api.Attribute;
import pl.edu.icm.unicore.uvos.api.Element;
import pl.edu.icm.unicore.uvos.api.Group;
import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.ModificationEvent;
import pl.edu.icm.unicore.uvos.api.exceptions.InvalidValueException;
import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.UVOSFault;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.WSQueryHistoryInterface;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.XMLBeansMapper;
import xmlbeans.pl.edu.icm.uvos.internal.AttributeXType;
import xmlbeans.pl.edu.icm.uvos.internal.ElementXType;
import xmlbeans.pl.edu.icm.uvos.internal.GroupXType;
import xmlbeans.pl.edu.icm.uvos.internal.IdentityXType;
import xmlbeans.pl.edu.icm.uvos.internal.ModificationEventXType;

/**
 * @author K. Benedyczak
 */
public class WSQueryHistoryClient implements UVOSQueryHistoryExtInterface
{
	private WSQueryHistoryInterface proxy;
	
	public WSQueryHistoryClient(WSQueryHistoryInterface proxy)
	{
		this.proxy = proxy;
	}

	public boolean areEquivalent(Identity i1, Identity i2, Date when) throws UVOSException
	{
		try
		{
			return proxy.areEquivalent(XMLBeansMapper.map2XMLIdentityDoc(i1), 
					XMLBeansMapper.map2XMLIdentityDoc(i2), when);
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
	}

	public List<Attribute> getAttributes(Element owner, String attribute, boolean effective,
			boolean includeScoped, boolean includeImplied, Date when) 
		throws UVOSException
	{
		AttributeXType[] raw;
		try
		{
			String attrName = null;
			if (attribute != null)
				attrName = attribute.toString();
			raw = proxy.getAttributes(
				XMLBeansMapper.map2XMLElementDoc(owner), 
				attrName,
				effective,
				includeScoped, 
				includeImplied, when).
				getAttributesList().getElementArray();
		} catch (UVOSFault e1)
		{
			throw e1.getAsAPIException();
		}
		ArrayList<Attribute> ret = new ArrayList<Attribute>();
		for (AttributeXType axt: raw)
			try
			{
				ret.add(XMLBeansMapper.map2APIAttribute(axt));
			} catch (XmlException e)
			{
				throw new InvalidValueException(e.getMessage());
			}
		return ret;
	}

	public List<Identity> getAllEquivalents(Identity who, Date when) throws UVOSException
	{
		IdentityXType[] raw;
		try
		{
			raw = proxy.getAllEquivalents(
					XMLBeansMapper.map2XMLIdentityDoc(who), when).
					getIdentitiesList().getElementArray();
		} catch (UVOSFault e1)
		{
			throw e1.getAsAPIException();
		}
		ArrayList<Identity> ret = new ArrayList<Identity>();
		for (IdentityXType ixt: raw)
			ret.add(XMLBeansMapper.map2APIIdentity(ixt));
		return ret;
	}

	public List<Group> getAllGroups(Identity who, boolean implied, Date when) throws UVOSException
	{
		GroupXType[] raw;
		try
		{
			raw = proxy.getAllGroups(
				XMLBeansMapper.map2XMLIdentityDoc(who),	implied, when).
				getGroupsList().getElementArray();
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
		ArrayList<Group> ret = new ArrayList<Group>();
		for (GroupXType ixt: raw)
			ret.add(XMLBeansMapper.map2APIGroup(ixt));
		return ret;
	}

	public List<Identity> getAllIdentities(Date when) throws UVOSException
	{
		IdentityXType[] raw;
		try
		{
			raw = proxy.getAllIdentities(when).getIdentitiesList().
				getElementArray();
		} catch (UVOSFault e1)
		{
			throw e1.getAsAPIException();
		}
		ArrayList<Identity> ret = new ArrayList<Identity>();
		for (IdentityXType ixt: raw)
			ret.add(XMLBeansMapper.map2APIIdentity(ixt));
		return ret;
	}

	public List<Element> getGroupContent(Group group, Date when) throws UVOSException
	{
		ElementXType[] raw;
		try
		{
			raw = proxy.getGroupContent(
					XMLBeansMapper.map2XMLGroupDoc(group),
					when).getElementsList().
					getElementArray();
		} catch (UVOSFault e1)
		{
			throw e1.getAsAPIException();
		} 
		
		ArrayList<Element> ret = new ArrayList<Element>();
		for (ElementXType xt: raw)
			ret.add(XMLBeansMapper.map2APIElement(xt));
		return ret;
	}

	public boolean isMember(Identity who, Group group, boolean effective, Date when) 
		throws UVOSException
	{
		try
		{
			return proxy.isMember(XMLBeansMapper.map2XMLIdentityDoc(who),
					XMLBeansMapper.map2XMLGroupDoc(group), effective, when);
		} catch (UVOSFault e)
		{
			throw e.getAsAPIException();
		}
	}

	public List<ModificationEvent> getHistoryEvents(Date from, Date to, 
			int categoryFilter, int typeFilter, boolean fillDetails) 
			throws UVOSException
	{
		ModificationEventXType[] raw;
		try
		{
			if (from == null)
				from = new Date(0);
			if (to == null)
			{
				Calendar hundredYearsLater = Calendar.getInstance();
				hundredYearsLater.add(Calendar.YEAR, 100);
				to = new Date(hundredYearsLater.getTimeInMillis());
			}
			raw = proxy.getHistoryEvents(from, to, categoryFilter, typeFilter,
				fillDetails).getModificationEventList().getElementArray();
		} catch (UVOSFault e1)
		{
			throw e1.getAsAPIException();
		} 
		
		ArrayList<ModificationEvent> ret = new ArrayList<ModificationEvent>();
		for (ModificationEventXType xt: raw)
			ret.add(XMLBeansMapper.map2APIModificationEvent(xt));
		return ret;
	}
}
