/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Sep 24, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.wsclient.samlapi;

import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.util.Properties;

import javax.security.auth.x500.X500Principal;

import org.apache.xmlbeans.XmlObject;

import eu.unicore.samly2.SAMLConstants;
import eu.unicore.samly2.elements.NameID;
import eu.unicore.security.util.client.IAuthenticationConfiguration;

import pl.edu.icm.unicore.uvos.wsclient.api.UVOSClientFactory;

/**
 * This class provides a base for SAML client implementations which use
 * SOAP binding. It adds issuer support, setups UVOS client factory and defines
 * generic methods.
 *
 * @author K. Benedyczak
 */
public abstract class SAMLVOClient extends AbstractSAMLBase
{
	protected String address;
	protected NameID issuer;
	protected UVOSClientFactory fact;
	
	protected XmlObject lastResponse;

	
	public SAMLVOClient(String address, IAuthenticationConfiguration secProv, Properties settings) 
		throws SecuritySetupException, MalformedURLException
	{
		this(address, secProv, (NameID)null, settings);
	}

	public SAMLVOClient(String address, IAuthenticationConfiguration secProv, 
			URI issuerURI, Properties settings) 
		throws SecuritySetupException, MalformedURLException
	{
		this(address, secProv, new NameID(issuerURI.toString(),
				SAMLConstants.NFORMAT_ENTITY), settings);
	}

	protected SAMLVOClient(String address, IAuthenticationConfiguration secCfg, 
			NameID issuer, Properties settings) 
		throws SecuritySetupException, MalformedURLException
	{
		super(secCfg);
		new URL(address);
		this.address = address;
		fact = new UVOSClientFactory(secCfg, settings);
		
		if (issuer != null)
			this.issuer = issuer;
		else
		{
			if (certificateC == null || certificateC.length == 0)
				throw new SecuritySetupException("No issuer was provided" +
						" and keystore doesn't provide user's identity" +
						" so it is impossible to generate one.", null);
			this.issuer = new NameID(
				certificateC[0].getSubjectX500Principal().
				getName(X500Principal.CANONICAL), 
				SAMLConstants.NFORMAT_DN);
		}
	}
	
	public XmlObject getLastResponse()
	{
		return lastResponse;
	}

	public abstract Object getProxy();
}
