/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Apr 12, 2008
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.uvos.test;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Date;
import java.util.List;

import junit.framework.TestCase;

import pl.edu.icm.unicore.uvos.api.Group;
import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.IdentityType;
import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;
import pl.edu.icm.unicore.uvos.client.util.ConfigHelper;
import pl.edu.icm.unicore.uvos.util.Configuration;
import pl.edu.icm.unicore.uvos.wsclient.api.UVOSAuthZExtInterface;
import pl.edu.icm.unicore.uvos.wsclient.api.UVOSClientFactory;
import pl.edu.icm.unicore.uvos.wsclient.api.UVOSManagementExtInterface;
import pl.edu.icm.unicore.uvos.wsclient.api.UVOSQueryExtInterface;

/**
 * @author K. Benedyczak
 */
public abstract class AbstractTestBase extends TestCase
{
	protected UVOSManagementExtInterface voMan;
	protected UVOSAuthZExtInterface voAuthz;
	protected UVOSQueryExtInterface voQuery;

	protected void connect(String configName) throws FileNotFoundException, IOException
	{				
		ConfigHelper conf = null;
		Configuration c = new Configuration(configName);
		conf = new ConfigHelper(c);
		
		String addr = conf.getProtocol() + "://" + 
			conf.getServerHost() + ":" + conf.getServerPort();
		UVOSClientFactory fact = new UVOSClientFactory(conf, conf.getConfiguration());
		
		voMan = fact.getWSManagementClientHL(addr);
		voAuthz = fact.getWSAuthZClientHL(addr);
		voQuery = fact.getWSQueryClientHL(addr);
	}
	
	protected String dumpList(List l)
	{
		if (l.size() == 0)
			return "EMPTY";
		String ret = l.get(0).toString();
		for (int i=1; i<l.size(); i++)
			ret = ret + ", " + l.get(i).toString();
		return ret;
	}
	
	/**
	 * createGroupStructure:
		/A
		--B/C
		--Z
	    createIdentities:
	    	2 equiv and one different (can be of the same type) idA1 idA2 and idB
	 */
	protected void createTestGI() throws UVOSException
	{
		gRoot = new Group("/");
		gA = voMan.addGroup(gRoot, "A");
		gB = voMan.addGroup(gA, "B");
		gC = voMan.addGroup(gB, "C");
		gZ = voMan.addGroup(gA, "Z");
		idA1 = new Identity(IdentityType.EMAIL, "a@a.com", "double");
		idA2 = new Identity(IdentityType.EMAIL, "agh@ex.org", "double");
		idB = new Identity(IdentityType.EMAIL, "bbb@some.dot", "single");
		voMan.addIdentity(idA1);
		voMan.addIdentity(idB);
		voMan.addIdentity(idA2, idA1);
	}
	
	protected void cleanTestGI() throws Exception
	{
		voMan.removeIdentity(idA1);
		voMan.removeIdentity(idA2);
		voMan.removeIdentity(idB);
		voMan.removeGroup(gA, true);
		voMan.purgeHistoricalData(new Date());
	}
	
	protected Group gRoot, gA, gB, gC, gZ;
	protected Identity idA1, idA2, idB;
}
