/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Sep 20, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.uvos.test;

import junit.framework.TestCase;

import pl.edu.icm.unicore.uvos.db.InitDB;
import pl.edu.icm.unicore.uvos.server.ComponentContainer;
import pl.edu.icm.unicore.uvos.server.VOServer;

/**
 * @author K. Benedyczak
 */
public class AllFunctionalTest extends TestCase
{
	private static final String UVOS_CONF = 
		"src/test/resources/uvos/uvosServer.conf";
	private static boolean initDone = false;
	
	@Override
	protected void setUp()
	{
		if (initDone)
			return;
		try
		{
			InitDB.main(new String[]{UVOS_CONF, "-b"});
			ComponentContainer container = new ComponentContainer(UVOS_CONF);
			VOServer serverInstance = container.getComponent(VOServer.class);
			serverInstance.startup(container.getContainer());
			initDone = true;
		} catch (Throwable e)
		{
			System.err.println("Severe error when starting UVOS Web Server:");
			e.printStackTrace();
			fail();
		}
	}
	
	
	public void test1Identities()
	{
		try
		{
			System.out.println("Doing test: Identities");
			IdentitiesFunctional test = new IdentitiesFunctional();
			test.connect("src/test/resources/functionalTest.conf");
			test.tstBasic();
			test.tstIdentities();
		} catch (Exception e)
		{
			e.printStackTrace();
			fail("Exception: " + e.getMessage());
		}
	}
	
	public void test2Groups()
	{
		try
		{
			System.out.println("Doing test: Groups");
			GroupFunctional test = new GroupFunctional();
			test.connect("src/test/resources/functionalTest.conf");
			test.tstGroups();
		} catch (Exception e)
		{
			e.printStackTrace();
			fail("Exception: " + e.getMessage());
		}
	}
	
	
	public void test3Attributes()
	{
		try
		{
			System.out.println("Doing test: Attributes");
			AttributeFunctional test = new AttributeFunctional();
			test.connect("src/test/resources/functionalTest.conf");
			test.testAll();
		} catch (Exception e)
		{
			e.printStackTrace();
			fail("Exception: " + e.getMessage());
		}
	}
	
	public void test4Composite()
	{
		try
		{
			System.out.println("Doing test: Composite");
			CompositeFunctional test = new CompositeFunctional();
			test.connect("src/test/resources/functionalTest.conf");
			test.testAll();
		} catch (Exception e)
		{
			e.printStackTrace();
			fail("Exception: " + e.getMessage());
		}
	}	
}

