/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Apr 12, 2008
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.uvos.test;

import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.IdentityType;
import pl.edu.icm.unicore.uvos.db.InitDB;
import pl.edu.icm.unicore.uvos.server.ComponentContainer;
import pl.edu.icm.unicore.uvos.server.VOServer;
import pl.edu.icm.unicore.uvos.util.Configuration;
import pl.edu.icm.unicore.uvos.wsclient.api.UVOSClientFactory;
import pl.edu.icm.unicore.uvos.wsclient.api.WSManagementClient;
import junit.framework.TestCase;

/**
 * AuthN tests with server configuration with failOnError=false 
 * @author K. Benedyczak
 */
public class AuthN3Test extends TestCase
{
	private static final String UVOS_CONF = "src/test/resources/uvos/uvosServerAuthn2.conf";

	private String addr;
	private TestSecurityConfig conf;
	private static boolean initDone = false;
	private ComponentContainer container;
	
	public AuthN3Test()
	{
		container = new ComponentContainer(UVOS_CONF);
	}
	
	@Override
	protected void setUp()
	{
		try
		{
			Configuration c = new Configuration("src/test/resources/functionalTest.conf");
			conf = new TestSecurityConfig(c);
		
			addr = conf.getProtocol() + "://" + 
				conf.getServerHost() + ":" + conf.getServerPort();
			
		} catch (Throwable e)
		{
			System.err.println("Severe error when starting UVOS Web Server:");
			e.printStackTrace();
			fail();
		}

		if (initDone)
			return;
		try
		{
			InitDB.main(new String[]{UVOS_CONF, "-b"});
			VOServer serverInstance = container.getComponent(VOServer.class);
			serverInstance.startup(container.getContainer());
			initDone = true;
		} catch (Throwable e)
		{
			System.err.println("Severe error when starting UVOS Web Server:");
			e.printStackTrace();
			fail();
		}

	}
	
	public void test1()
	{
		try
		{
			System.out.println("\nChecking authN failures in chain\n");
			conf.doSSLAuthn = true;
			conf.doHttpAuthn = true;
			conf.httpPasswd = ""; 
			conf.httpUser = "voadmin@localhost";
			UVOSClientFactory fact = new UVOSClientFactory(conf, conf.getConfiguration());
			WSManagementClient voMan = fact.getWSManagementClientHL(addr);
			voMan.getAttributeTypes();
		} catch (Exception e)
		{
			fail("Didn't get access with wrong SSL authN and correct HTTP" +
				" while failures should be ignored).");
			
		}
	}
	
	public void test2()
	{
		try
		{
			Identity testing = new Identity(IdentityType.DN, 
			"cn=example client,ou=example ou,o=example o,l=example city,st=example province,c=pl");

			conf.doSSLAuthn = true;
			conf.doHttpAuthn = true;
			conf.httpPasswd = ""; 
			conf.httpUser = "voadmin@localhost";
			UVOSClientFactory fact = new UVOSClientFactory(conf, conf.getConfiguration());
			WSManagementClient voMan = fact.getWSManagementClientHL(addr);
			voMan.addIdentity(testing);
		} catch (Exception e)
		{
			System.err.println("Error while adding an identity (directly)");
			e.printStackTrace();			
			fail();
		}

		try
		{
			System.out.println("\nChecking authN failures in chain 2\n");
			conf.doSSLAuthn = true;
			conf.doHttpAuthn = true;
			conf.httpPasswd = "wrong password"; 
			conf.httpUser = "voadmin@localhost";
			UVOSClientFactory fact = new UVOSClientFactory(conf, conf.getConfiguration());
			WSManagementClient voMan = fact.getWSManagementClientHL(addr);
			voMan.getAttributeTypes();
		} catch (Exception e)
		{
			fail("Didn't get access with correct SSL authN and " +
				"incorrect HTTP while the later should be ignored.");		
		}
	}
}
