/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Apr 12, 2008
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.uvos.test;

import junit.framework.TestCase;
import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;
import pl.edu.icm.unicore.uvos.db.InitDB;
import pl.edu.icm.unicore.uvos.server.ComponentContainer;
import pl.edu.icm.unicore.uvos.server.VOServer;
import pl.edu.icm.unicore.uvos.util.Configuration;
import pl.edu.icm.unicore.uvos.wsclient.api.UVOSClientFactory;
import pl.edu.icm.unicore.uvos.wsclient.api.WSManagementClient;

/**
 * AuthN tests with server configuration with failOnError=true and normal SSL authN
 * @author K. Benedyczak
 */
public class AuthN4Test extends TestCase
{
	private static final String UVOS_CONF = "src/test/resources/uvos/uvosServerAuthn1.conf";

	private static boolean initDone = false;
	protected String addr;
	protected TestSecurityConfig conf, confWrongCert;
	private ComponentContainer container;
	
	public AuthN4Test()
	{
		container = new ComponentContainer(UVOS_CONF);
	}
	
	@Override
	protected void setUp()
	{
		try
		{
			System.setProperty("crlmanager.properties.file", "src/test/resources/certs/crlcheck.properties");
			Configuration c = new Configuration("src/test/resources/functionalTest.conf");
			conf = new TestSecurityConfig(c);
			Configuration c2 = new Configuration("src/test/resources/authnTest1.conf");
			confWrongCert = new TestSecurityConfig(c2);
	
			addr = conf.getProtocol() + "://" + 
				conf.getServerHost() + ":" + conf.getServerPort();
		} catch (Throwable e)
		{
			System.err.println("Severe error when starting UVOS Web Server:");
			e.printStackTrace();
			fail();
		}

		if (initDone)
			return;
		try
		{
			InitDB.main(new String[]{UVOS_CONF, "-b"});
			VOServer serverInstance = container.getComponent(VOServer.class);
			serverInstance.startup(container.getContainer());
		} catch (Throwable e)
		{
			System.err.println("Severe error when starting UVOS Web Server:");
			e.printStackTrace();
			fail();
		}
		initDone = true;
	}
	
	public void testSSLUnknown()
	{
		try
		{
			System.out.println("\nChecking access with SSL authN while identity DN is unknown\n");
			conf.doSSLAuthn = true;
			UVOSClientFactory fact = new UVOSClientFactory(conf, conf.getConfiguration());
			WSManagementClient voMan = fact.getWSManagementClientHL(addr);
			voMan.getAttributeTypes();
			System.out.println("Got access with SSL authN while identity should be unknown.");
			fail("Got access with SSL authN while identity should be unknown.");
		} catch (UVOSException e){}
		catch (Exception e)
		{
			e.printStackTrace();
			fail("Wrong error: " + e);
		}
	}
	
	public void testAuthnChainFailures()
	{
		try
		{
			System.out.println("\nChecking authN failures in chain\n");
			conf.doSSLAuthn = true;
			conf.doHttpAuthn = true;
			conf.httpPasswd = "";
			conf.httpUser = "admin@localhost";
			UVOSClientFactory fact = new UVOSClientFactory(conf, conf.getConfiguration());
			WSManagementClient voMan = fact.getWSManagementClientHL(addr);
			voMan.getAttributeTypes();
			System.out.println("Got access with SSL authN while identity should be unknown " +
			"(correct HTTP authN was given but should be ignored).");
			fail("Got access with SSL authN while identity should be unknown " +
				"(correct HTTP authN was given but should be ignored).");
		} catch (Exception e){}
	}
}
