/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Apr 12, 2008
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.uvos.test;

import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.IdentityType;
import pl.edu.icm.unicore.uvos.api.Permissions;
import pl.edu.icm.unicore.uvos.db.InitDB;
import pl.edu.icm.unicore.uvos.engine.VirtualSecurityContextImpl;
import pl.edu.icm.unicore.uvos.engine.api.UVOSManagementInterface;
import pl.edu.icm.unicore.uvos.server.ComponentContainer;
import pl.edu.icm.unicore.uvos.server.VOServer;
import pl.edu.icm.unicore.uvos.server.idresolver.Authenticator;
import pl.edu.icm.unicore.uvos.util.Configuration;
import pl.edu.icm.unicore.uvos.wsclient.api.UVOSClientFactory;
import pl.edu.icm.unicore.uvos.wsclient.api.WSManagementClient;
import junit.framework.TestCase;

/**
 * Basic and HTTP authN tests.
 * @author K. Benedyczak
 */
public class AuthNTest extends TestCase
{
	private static final String UVOS_CONF = 
		"src/test/resources/uvos/uvosServer.conf";
	private static boolean initDone = false;

	private String addr;
	private TestSecurityConfig conf;
	private ComponentContainer container;
	
	@Override
	protected void setUp()
	{
		try
		{
			conf = null;
			Configuration c = new Configuration("src/test/resources/functionalTest.conf");
			conf = new TestSecurityConfig(c);
			
			addr = conf.getProtocol() + "://" + 
				conf.getServerHost() + ":" + conf.getServerPort();
		} catch (Throwable e)
		{
			System.err.println("Severe error when starting UVOS Web Server:");
			e.printStackTrace();
			fail();
		}

		if (initDone)
			return;
		try
		{
			InitDB.main(new String[]{UVOS_CONF, "-b"});
			container = new ComponentContainer(UVOS_CONF);
			VOServer serverInstance = container.getComponent(VOServer.class);
			serverInstance.startup(container.getContainer());
			UVOSManagementInterface man = container.getComponent(UVOSManagementInterface.class); 
			Identity testing = new Identity(IdentityType.EMAIL, 
					"test@withPasswd.org");
			testing.setAuthnData("secret");
			Authenticator.injectFakeContext(new VirtualSecurityContextImpl(Permissions.ALL_P));
			man.addIdentity(testing);
			Authenticator.injectFakeContext(null);
			initDone = true;
		} catch (Throwable e)
		{
			System.err.println("Severe error when starting UVOS Web Server:");
			e.printStackTrace();
			fail();
		}
	}
	
	public void testHTTPPasswd()
	{
		try
		{
			System.out.println("\nTesting access with correct HTTP data\n");
			conf.doSSLAuthn = false;
			conf.doHttpAuthn = true;
			conf.httpPasswd = "secret"; 
			conf.httpUser = "test@withPasswd.org";
			UVOSClientFactory fact = new UVOSClientFactory(conf, conf.getConfiguration());
			
			WSManagementClient voMan = fact.getWSManagementClientHL(addr);
			voMan.getAttributeTypes();
		} catch (Exception e)
		{
			fail("Didn't get access with correct HTTP data: " + e);
		}
	}

	public void testWrongHTTPPasswd()
	{
		try
		{
			System.out.println("\nTesting access with wrong HTTP passwd\n");
			conf.doSSLAuthn = false;
			conf.doHttpAuthn = true;
			conf.httpPasswd = "secretddd"; 
			conf.httpUser = "test@withPasswd.org";
			UVOSClientFactory fact = new UVOSClientFactory(conf, conf.getConfiguration());
			WSManagementClient voMan = fact.getWSManagementClientHL(addr);
			voMan.getAttributeTypes();
			fail("Got access with incorrect HTTP data");
		} catch (Exception e)
		{
		}
	}

	public void testWrongHTTPUser()
	{
		try
		{
			System.out.println("\nTesting access with wrong HTTP user\n");
			conf.doSSLAuthn = false;
			conf.doHttpAuthn = true;
			conf.httpPasswd = "secret";
			conf.httpUser = "none@withPasswd.org";
			UVOSClientFactory fact = new UVOSClientFactory(conf, conf.getConfiguration());
			WSManagementClient voMan = fact.getWSManagementClientHL(addr);
			voMan.getAttributeTypes();
			fail("Got access with incorrect HTTP data");
		} catch (Exception e)
		{
		}
	}
	
	public void testHTTPPasswdWrongCert()
	{
		try
		{
			System.out.println("\nTesting access with correct HTTP data and wrong cert\n");
			conf.doSSLAuthn = true;
			conf.doHttpAuthn = true;
			conf.httpPasswd = "secret"; 
			conf.httpUser = "test@withPasswd.org";
			UVOSClientFactory fact = new UVOSClientFactory(conf, conf.getConfiguration());
			WSManagementClient voMan = fact.getWSManagementClientHL(addr);
			voMan.getAttributeTypes();
		} catch (Exception e)
		{
			fail("Didn't get access with correct HTTP data: " + e);
		}
	}

	public void testWrongHTTPPasswdGoodCert()
	{
		try
		{
			System.out.println("\nTesting access with wrong HTTP passwd and good cert\n");
			conf.doSSLAuthn = true;
			conf.doHttpAuthn = true;
			conf.httpPasswd = "secretkkk"; 
			conf.httpUser = "test@withPasswd.org";
			UVOSClientFactory fact = new UVOSClientFactory(conf, conf.getConfiguration());
			WSManagementClient voMan = fact.getWSManagementClientHL(addr);
			voMan.getAttributeTypes();
			fail("Got access with incorrect HTTP data");
		} catch (Exception e)
		{
		}
	}

}
