/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Sep 20, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.uvos.test;

import java.util.List;

import pl.edu.icm.unicore.uvos.api.Element;

/**
 * Performs functional tests. Assumptions:
 * Server is up and running and DB is empty
 * @author K. Benedyczak
 */
public class CompositeFunctional extends AbstractTestBase
{
	public void testAll() throws Exception
	{
		createTestGI();
		tstAddToGroup();
		tstRemoveFromGroup();
		cleanTestGI();
	}
	
	private void tstAddToGroup() throws Exception
	{
		voMan.addToGroup(idA1, gC);
		voMan.addToGroup(idB, gA);
		List<Element> c = voQuery.getGroupContent(gC);
		if (c.size() != 2)
			fail("Group " + gC + " should have two elements, has " + c.size());
		Element e = c.get(0);
		if (e.getGroup() != null || 
			!(e.getIdentity().equals(idA1) || e.getIdentity().equals(idA2)))
			fail("Group " + gC + " should contain " + idA1 + 
					" or " + idA2 + " but contains " + e);
		c = voQuery.getGroupContent(gA);
		if (c.size() != 3)
			fail("Group " + gA + " should have three elements, has " + c.size());
		e = c.get(0);
		if (e.getIdentity() != null && !e.getIdentity().equals(idB))
			fail("Identity in group " + gA + " should be " + idB + 
					" but is " + e);
	}

	private void tstRemoveFromGroup() throws Exception
	{
		voMan.removeFromGroup(idA1, gC);
		voMan.removeFromGroup(idB, gA);
		List<Element> c = voQuery.getGroupContent(gC);
		if (c.size() != 0)
			fail("Group " + gC + " should have no elements, has " + c.size());
		c = voQuery.getGroupContent(gA);
		if (c.size() != 2)
			fail("Group " + gA + " should have two elements, has " + c.size());		
	}
}














