/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Apr 12, 2008
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.uvos.test;

import java.io.FileNotFoundException;
import java.io.IOException;

import eu.unicore.samly2.SAMLConstants;

import junit.framework.TestCase;
import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.IdentityType;
import pl.edu.icm.unicore.uvos.client.util.ConfigHelper;
import pl.edu.icm.unicore.uvos.db.InitDB;
import pl.edu.icm.unicore.uvos.server.ComponentContainer;
import pl.edu.icm.unicore.uvos.server.VOServer;
import pl.edu.icm.unicore.uvos.util.Configuration;
import pl.edu.icm.unicore.uvos.wsclient.api.UVOSClientFactory;
import pl.edu.icm.unicore.uvos.wsclient.api.UVOSManagementExtInterface;
import pl.edu.icm.unicore.uvos.wsclient.samlapi.SAMLErrorResponseException;
import pl.edu.icm.unicore.uvos.wsclient.samlapi.SAMLVOQueryClient;

/**
 * Test if identity status (disabled/enabled) is in effect. 
 * @author K. Benedyczak
 */
public class IdentityStatusTest extends TestCase
{
	private static final String UVOS_CONF = "src/test/resources/uvos/uvosServer.conf";

	private static boolean initDone = false;
	protected String addr;
	protected TestSecurityConfig conf;
	protected UVOSManagementExtInterface voMan;
	
	@Override
	protected void setUp()
	{
		try
		{
			Configuration c = new Configuration("src/test/resources/functionalTest.conf");
			conf = new TestSecurityConfig(c);
			
			addr = conf.getProtocol() + "://" + 
				conf.getServerHost() + ":" + conf.getServerPort();
			connect(c);
		} catch (Throwable e)
		{
			System.err.println("Severe error when creating configuration");
			e.printStackTrace();
			fail();
		}

		if (initDone)
			return;
		try
		{
			InitDB.main(new String[]{UVOS_CONF, "-b"});
			ComponentContainer container = new ComponentContainer(UVOS_CONF);
			VOServer serverInstance = container.getComponent(VOServer.class);
			serverInstance.startup(container.getContainer());
		} catch (Throwable e)
		{
			System.err.println("Severe error when starting UVOS Web Server:");
			e.printStackTrace();
			fail();
		}
		initDone = true;
	}
	
	protected void connect(Configuration c) throws FileNotFoundException, IOException
	{				
		ConfigHelper manConf = new ConfigHelper(c);
		
		String addr = manConf.getProtocol() + "://" + 
			manConf.getServerHost() + ":" + manConf.getServerPort();
		UVOSClientFactory fact = new UVOSClientFactory(manConf, c);
		
		voMan = fact.getWSManagementClientHL(addr);
	}

	
	public void test()
	{
		try
		{
			Identity tested = new Identity(IdentityType.EMAIL, "a@b");
			voMan.addIdentity(tested);
			
			
			conf.doHttpAuthn = true;
			conf.httpPasswd = "";
			conf.httpUser = "a@b";
			conf.doSSLAuthn = false;
			SAMLVOQueryClient client = new SAMLVOQueryClient(addr, conf, conf.getConfiguration());
			//this tests both authentication and SAML query subject (both should succeed) 
			client.getAttributes(tested);
			
			voMan.setIdentityStatus(tested, false);
			
			try
			{
				client.getAttributes(tested);
				System.err.println("Can do SAML selfquery for disabled user");
				fail("Can do SAML selfquery for disabled user");
			} catch (SAMLErrorResponseException e)
			{
				//OK but the status should be also AUTHN_FAILED 
				if (!e.getSubStatus().equals(SAMLConstants.STATUS2_AUTHN_FAILED))
				{
					String m = "Wrong error status for" +
					" SAML selfquery for disabled user: " + e.getSubStatus();
					System.err.println(m);
					fail(m);
				}
			}
			
			conf.doHttpAuthn = true;
			conf.httpPasswd = "";
			conf.httpUser = "voadmin@localhost";
			conf.doSSLAuthn = false;
			client = new SAMLVOQueryClient(addr, conf, conf.getConfiguration());
			
			try
			{
				client.getAttributes(tested);
				System.err.println("Can do SAML query for disabled user");
				fail("Can do SAML query for disabled user");
			} catch (SAMLErrorResponseException e)
			{
				//OK but the status should be also UNKNOWN_PRINCIPIAL 
				if (!e.getSubStatus().equals(SAMLConstants.STATUS2_UNKNOWN_PRINCIPIAL))
				{
					String m = "Wrong error status for" +
						" SAML query for disabled user: " + e.getSubStatus();
					System.err.println(m);
					fail(m);
				}
			}
			
			voMan.setIdentityStatus(tested, true);
			client.getAttributes(tested);
			
			
		} catch (Exception e)
		{
			e.printStackTrace();
			fail("User was not authenticated");
		}
		
	}
}
