/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Apr 12, 2008
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.uvos.test;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import eu.unicore.bugsreporter.annotation.RegressionTest;
import eu.unicore.samly2.assertion.Assertion;
import eu.unicore.samly2.exceptions.SAMLParseException;

import junit.framework.TestCase;
import pl.edu.icm.unicore.uvos.api.Attribute;
import pl.edu.icm.unicore.uvos.api.Element;
import pl.edu.icm.unicore.uvos.api.Group;
import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.IdentityType;
import pl.edu.icm.unicore.uvos.api.exceptions.AttributeNotFoundException;
import pl.edu.icm.unicore.uvos.client.util.ConfigHelper;
import pl.edu.icm.unicore.uvos.db.InitDB;
import pl.edu.icm.unicore.uvos.server.ComponentContainer;
import pl.edu.icm.unicore.uvos.server.VOServer;
import pl.edu.icm.unicore.uvos.util.AttributeFilter;
import pl.edu.icm.unicore.uvos.util.Configuration;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.SAMLXMLBeansMapper;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.UnsupportedSAMLException;
import pl.edu.icm.unicore.uvos.wsclient.api.UVOSClientFactory;
import pl.edu.icm.unicore.uvos.wsclient.api.UVOSManagementExtInterface;
import pl.edu.icm.unicore.uvos.wsclient.samlapi.SAMLVOQueryClient;

/**
 * @author K. Benedyczak
 */
public class SAMLAttribute2Test extends TestCase
{
	private static final String UVOS_CONF = "src/test/resources/uvos/uvosServer.conf";

	private static boolean initDone = false;
	protected TestSecurityConfig conf;
	protected String addr;
	protected UVOSManagementExtInterface voMan;

	protected static Identity tested = new Identity(IdentityType.DN, 
		"CN=Example Client, OU=Example OU, O=Example O, L=Example City, ST=Example Province, C=PL");
	
	@Override
	protected void setUp()
	{
		try
		{
			Configuration c = new Configuration("src/test/resources/tlsTest.conf");
			conf = new TestSecurityConfig(c);
			addr = conf.getProtocol() + "://" + 
				conf.getServerHost() + ":" + conf.getServerPort();
			Configuration c2 = new Configuration("src/test/resources/functionalTest.conf");
			connectMan(c2);
		} catch (Throwable e)
		{
			System.err.println("Severe error when creating configuration");
			e.printStackTrace();
			fail();
		}

		if (initDone)
			return;
		try
		{
			InitDB.main(new String[]{UVOS_CONF, "-b"});
			ComponentContainer container = new ComponentContainer(UVOS_CONF);
			VOServer serverInstance = container.getComponent(VOServer.class);
			serverInstance.startup(container.getContainer());
			
			initAttributes();
		} catch (Throwable e)
		{
			System.err.println("Severe error when starting UVOS Web Server:");
			e.printStackTrace();
			fail();
		}
		initDone = true;
	}
	
	protected void connectMan(Configuration c) throws FileNotFoundException, IOException
	{				
		ConfigHelper manConf = new ConfigHelper(c);
		
		String addr = manConf.getProtocol() + "://" + 
			manConf.getServerHost() + ":" + manConf.getServerPort();
		UVOSClientFactory fact = new UVOSClientFactory(manConf, c);
		
		voMan = fact.getWSManagementClientHL(addr);
	}

	/**
	 * Adding attributes: 3 global (null value, 1 value, 2 values)
	 * and 3 scoped in /g/sub (where user is a member).
	 * 
	 * Therefore user receives the following effective attributes within different scopes:
	 *  3 global = 3
	 *  3 global x 2 scopes each = 6
	 *  3 scoped x 2 scopes each = 6
	 *  1 group attribute = 1
	 *  -----------------------------
	 *  16
	 */
	private void initAttributes() throws Exception
	{
		voMan.addIdentity(tested);
		
		Group g = new Group("/g");
		voMan.addGroup(null, g.getName());
		Group scope = voMan.addGroup(g, "sub");
		
		voMan.addToGroup(tested, scope);
		

		voMan.setAttribute(new Element(null, tested), new Attribute("global1", 
			null, null), false);
		voMan.setAttribute(new Element(null, tested), new Attribute("global2", 
			Collections.singletonList("v3"), null), false);
		List<String> vals = new ArrayList<String>();
		vals.add("v1");
		vals.add("v2");
		voMan.setAttribute(new Element(null, tested), new Attribute("global3", 
			vals, null), false);
		
		voMan.setAttribute(new Element(scope, tested), new Attribute("scoped1", 
			null, null), false);
		voMan.setAttribute(new Element(scope, tested), new Attribute("scoped2", 
			Collections.singletonList("v3"), null), false);
		voMan.setAttribute(new Element(scope, tested), new Attribute("scoped3", 
			vals, null), false);
	}
	
	private static class FilterAll implements AttributeFilter
	{
		@Override
		public List<Attribute> filter(List<Attribute> attribs)
		{
			return attribs;
		}
		@Override
		public boolean allowsEverything()
		{
			return false;
		}
	}

	private static class FilterNone implements AttributeFilter
	{
		@Override
		public List<Attribute> filter(List<Attribute> attribs)
		{
			return Collections.emptyList();
		}
		@Override
		public boolean allowsEverything()
		{
			return false;
		}
	}
	private static class FilterG1S2 implements AttributeFilter
	{
		@Override
		public List<Attribute> filter(List<Attribute> attribs)
		{
			
			List<Attribute> ret = new ArrayList<Attribute>(); 
			List<String> empty = new ArrayList<String>();
			ret.add(new Attribute("global1", empty, null));
			ret.add(new Attribute("global1", empty, "/g"));
			ret.add(new Attribute("global1", empty, "/g/sub"));
			ret.add(new Attribute("scoped2", 
				Collections.singletonList("v3"), "/g"));
			ret.add(new Attribute("scoped2", 
				Collections.singletonList("v3"), "/g/sub"));
			return ret;
		}
		@Override
		public boolean allowsEverything()
		{
			return false;
		}
	}
	private static class FilterG3 implements AttributeFilter
	{
		@Override
		public List<Attribute> filter(List<Attribute> attribs)
		{
			List<Attribute> ret = new ArrayList<Attribute>(); 
			List<String> vals = new ArrayList<String>();
			vals.add("v1");
			vals.add("v2");
			ret.add(new Attribute("global3", vals, null));
			ret.add(new Attribute("global3", vals, "/g"));
			ret.add(new Attribute("global3", vals, "/g/sub"));
			return ret;
		}
		@Override
		public boolean allowsEverything()
		{
			return false;
		}
	}
	
	@RegressionTest(url="https://sourceforge.net/tracker/?func=detail&aid=3479285&group_id=102081&atid=1437373")
	public void testInteropAttributeFiltering()
	{
		SAMLVOQueryClient client;
		try
		{
			client = new SAMLVOQueryClient(addr, conf, conf.getConfiguration());
			client.setDSigPolicy(true, true, null);
			
			List<String> grpVals = new ArrayList<String>();
			Collections.addAll(grpVals, "/g/sub", "/g");
			Attribute grA = new Attribute("urn:SAML:voprofile:group", grpVals, null);
			
			List<String> empty = new ArrayList<String>();
			Attribute g1 = new Attribute("global1", empty, null);
			Attribute g2 = new Attribute("global2",	
				Collections.singletonList("v3"), null); 
			List<String> vals = new ArrayList<String>();
			vals.add("v1");
			vals.add("v2");
			Attribute g3 = new Attribute("global3",	vals, null);

			Attribute g1g = new Attribute("global1", empty, "/g");
			Attribute g2g = new Attribute("global2",	
				Collections.singletonList("v3"), "/g"); 
			Attribute g3g = new Attribute("global3", vals, "/g");

			Attribute g1gs = new Attribute("global1", empty, "/g/sub");
			Attribute g2gs = new Attribute("global2",	
				Collections.singletonList("v3"), "/g/sub"); 
			Attribute g3gs = new Attribute("global3", vals, "/g/sub");
			
			Attribute s1 = new Attribute("scoped1", empty, "/g/sub");
			Attribute s2 = new Attribute("scoped2", 
				Collections.singletonList("v3"), "/g/sub");
			Attribute s3 = new Attribute("scoped3",	vals, "/g/sub");
			Attribute s1g = new Attribute("scoped1", empty, "/g");
			Attribute s2g = new Attribute("scoped2", 
				Collections.singletonList("v3"), "/g");
			Attribute s3g = new Attribute("scoped3", vals, "/g");

			Set<Attribute> all = new HashSet<Attribute>();
			Collections.addAll(all, g1, g2, g3, s1, s2, s3, 
				g1g, g2g, g3g, g1gs, g2gs, g3gs,
				s1g, s2g, s3g, grA);
			
			Set<Attribute> allG = new HashSet<Attribute>();
			Collections.addAll(allG, g1, g2, g3, s1g, s2g, s3g, g1g, g2g, g3g, grA);
			
			Set<Attribute> setG1S2 = new HashSet<Attribute>();
			Collections.addAll(setG1S2, g1, g1g, g1gs, s2, s2g);

			Set<Attribute> setG1S2g = new HashSet<Attribute>();
			Collections.addAll(setG1S2g, g1, g1g, s2g);

			Set<Attribute> setG3 = new HashSet<Attribute>();
			Collections.addAll(setG3, g3, g3g, g3gs);

			Set<Attribute> setG3g = new HashSet<Attribute>();
			Collections.addAll(setG3g, g3, g3g);
			
			try
			{
				client.getFilteredAssertion(tested, null, true, new FilterNone());
				fail("Empty filtering succeeded");
			} catch (AttributeNotFoundException e)
			{
				//OK - expected
			}
			
			Assertion a1 = client.getFilteredAssertion(tested, null, false, null);
			compare(a1, all);
			
			Assertion a2 = client.getFilteredAssertion(tested, new Group("/g"), false, null);
			compare(a2, allG);
			
			Assertion a3 = client.getFilteredAssertion(tested, new Group("/g"), true, null);
			compare(a3, all);
			
			Assertion a4 = client.getFilteredAssertion(tested, null, true, new FilterAll());
			compare(a4, all);
			

			Assertion a6 = client.getFilteredAssertion(tested, null, true, new FilterG1S2());
			compare(a6, setG1S2);
			
			Assertion a7 = client.getFilteredAssertion(tested, null, true, new FilterG3());
			compare(a7, setG3);
			
			Assertion a8 = client.getFilteredAssertion(tested, new Group("/g"), false, new FilterG1S2());
			compare(a8, setG1S2g);
			
			Assertion a9 = client.getFilteredAssertion(tested, new Group("/g"), false, new FilterG3());
			compare(a9, setG3g);
		} catch (Exception e)
		{
			e.printStackTrace();
			fail("Error: " + e.toString());
		}
	}
	
	private void compare(Assertion a, Set<Attribute> model) throws UnsupportedSAMLException, SAMLParseException
	{
		List<Attribute> received = SAMLXMLBeansMapper.convertSAMLAttributes2API(a);
		for (Attribute attr: received)
		{
			if (!model.contains(attr))
				fail("Received attribute which was not expected: " + attr);
		}
		assertEquals("Got less attributes then expected", model.size(), received.size());
	}
	
	
	public void testAll()
	{
		try
		{
			System.out.println(conf.doSSLAuthn);
			SAMLVOQueryClient client = new SAMLVOQueryClient(addr, conf, conf.getConfiguration());
			client.setDSigPolicy(true, true, null);
			
			List<Attribute> attrs = client.getAttributes(tested);
			if (attrs.size() != 16)
				fail("Should get 15 attributes, got " + attrs.size() + ": " + attrs);
		} catch (Exception e)
		{
			e.printStackTrace();
			fail("User was not authenticated");
		}
	}

	public void testIndividualOneValue()
	{
		try
		{
			System.out.println(conf.doSSLAuthn);
			SAMLVOQueryClient client = new SAMLVOQueryClient(addr, conf, conf.getConfiguration());
			client.setDSigPolicy(true, true, null);
			
			List<Attribute> attrs = client.getAttributes(tested, 
				new Attribute("global1", Collections.singletonList("v1"), null));
			if (attrs.size() != 0)
				fail("Should get 0 attributes, got " + attrs.size() + ": " + attrs);
			
			attrs = client.getAttributes(tested, 
				new Attribute("global2", Collections.singletonList("v3"), null));
			if (attrs.size() != 3)
				fail("Should get 3 attributes, got " + attrs.size() + ": " + attrs);

			attrs = client.getAttributes(tested, 
				new Attribute("global3", Collections.singletonList("v1"), null));
			if (attrs.size() != 3)
				fail("Should get 3 attributes, got " + attrs.size() + ": " + attrs);

			attrs = client.getAttributes(tested, 
				new Attribute("scoped1", Collections.singletonList("v1"), null));
			if (attrs.size() != 0)
				fail("Should get 0 attributes, got " + attrs.size() + ": " + attrs);

			attrs = client.getAttributes(tested, 
				new Attribute("scoped2", Collections.singletonList("v3"), null));
			if (attrs.size() != 2)
				fail("Should get 2 attributes, got " + attrs.size() + ": " + attrs);

			attrs = client.getAttributes(tested, 
				new Attribute("scoped3", Collections.singletonList("v1"), null));
			if (attrs.size() != 2)
				fail("Should get 2 attributes, got " + attrs.size() + ": " + attrs);
		} catch (Exception e)
		{
			e.printStackTrace();
			fail("User was not authenticated");
		}
	}


	public void testIndividualAllValues()
	{
		try
		{
			System.out.println(conf.doSSLAuthn);
			SAMLVOQueryClient client = new SAMLVOQueryClient(addr, conf, conf.getConfiguration());
			client.setDSigPolicy(true, true, null);
			
			List<Attribute> attrs = client.getAttributes(tested, 
				new Attribute("global1", null, null));
			if (attrs.size() != 3)
				fail("Should get 3 attributes, got " + attrs.size() + ": " + attrs);
			
			attrs = client.getAttributes(tested, 
				new Attribute("global2", null, null));
			if (attrs.size() != 3)
				fail("Should get 3 attributes, got " + attrs.size() + ": " + attrs);

			attrs = client.getAttributes(tested, 
				new Attribute("global3", null, null));
			if (attrs.size() != 3)
				fail("Should get 3 attributes, got " + attrs.size() + ": " + attrs);

			attrs = client.getAttributes(tested, 
				new Attribute("scoped1", null, null));
			if (attrs.size() != 2)
				fail("Should get 2 attributes, got " + attrs.size() + ": " + attrs);

			attrs = client.getAttributes(tested, 
				new Attribute("scoped2", null, null));
			if (attrs.size() != 2)
				fail("Should get 2 attributes, got " + attrs.size() + ": " + attrs);

			attrs = client.getAttributes(tested, 
				new Attribute("scoped3", null, null));
			if (attrs.size() != 2)
				fail("Should get 2 attributes, got " + attrs.size() + ": " + attrs);
		} catch (Exception e)
		{
			e.printStackTrace();
			fail("User was not authenticated");
		}
	}


	public void testScoped()
	{
		try
		{
			System.out.println(conf.doSSLAuthn);
			SAMLVOQueryClient client = new SAMLVOQueryClient(addr, conf, conf.getConfiguration());
			client.setDSigPolicy(true, true, null);
			
			List<Attribute> attrs = client.getAttributes(tested, 
				new Group("/g"));
			if (attrs.size() != 7)
				fail("Should get 7 attributes, got " + attrs.size() + ": " + attrs);

			attrs = client.getAttributes(tested, 
				new Group("/g/sub"));
			if (attrs.size() != 7)
				fail("Should get 7 attributes, got " + attrs.size() + ": " + attrs);
		} catch (Exception e)
		{
			e.printStackTrace();
			fail("User was not authenticated");
		}
	}



}
