/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Apr 12, 2008
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.uvos.test;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import eu.unicore.samly2.assertion.Assertion;

import junit.framework.TestCase;
import pl.edu.icm.unicore.uvos.api.Attribute;
import pl.edu.icm.unicore.uvos.api.Element;
import pl.edu.icm.unicore.uvos.api.Group;
import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.IdentityType;
import pl.edu.icm.unicore.uvos.client.util.ConfigHelper;
import pl.edu.icm.unicore.uvos.db.InitDB;
import pl.edu.icm.unicore.uvos.idhelpers.X509IdentityHelper;
import pl.edu.icm.unicore.uvos.server.ComponentContainer;
import pl.edu.icm.unicore.uvos.server.VOServer;
import pl.edu.icm.unicore.uvos.util.Configuration;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.SAMLXMLBeansMapper;
import pl.edu.icm.unicore.uvos.wsclient.api.UVOSClientFactory;
import pl.edu.icm.unicore.uvos.wsclient.api.UVOSManagementExtInterface;
import pl.edu.icm.unicore.uvos.wsclient.samlapi.SAMLVOQueryClient;
import xmlbeans.org.oasis.saml2.assertion.AttributeStatementType;
import xmlbeans.org.oasis.saml2.assertion.AttributeType;

/**
 * @author K. Benedyczak
 */
public class SAMLAttributeTest extends TestCase
{
	private static final String UVOS_CONF = "src/test/resources/uvos/uvosServer.conf";

	private static boolean initDone = false;
	protected String addr;
	protected TestSecurityConfig conf;
	protected UVOSManagementExtInterface voMan;
	
	@Override
	protected void setUp()
	{
		try
		{
			Configuration c = new Configuration("src/test/resources/functionalTest.conf");
			conf = new TestSecurityConfig(c);
			
			addr = conf.getProtocol() + "://" + 
				conf.getServerHost() + ":" + conf.getServerPort();
			connect(c);
		} catch (Throwable e)
		{
			System.err.println("Severe error when creating configuration");
			e.printStackTrace();
			fail();
		}

		if (initDone)
			return;
		try
		{
			InitDB.main(new String[]{UVOS_CONF, "-b"});
			ComponentContainer container = new ComponentContainer(UVOS_CONF);
			VOServer serverInstance = container.getComponent(VOServer.class);
			serverInstance.startup(container.getContainer());
		} catch (Throwable e)
		{
			System.err.println("Severe error when starting UVOS Web Server:");
			e.printStackTrace();
			fail();
		}
		initDone = true;
	}
	
	protected void connect(Configuration c) throws FileNotFoundException, IOException
	{				
		ConfigHelper manConf = new ConfigHelper(c);
		
		String addr = manConf.getProtocol() + "://" + 
			manConf.getServerHost() + ":" + manConf.getServerPort();
		UVOSClientFactory fact = new UVOSClientFactory(manConf, c);
		
		voMan = fact.getWSManagementClientHL(addr);
	}

	public void testGroupAsAttributes()
	{
		try
		{
			Identity tested = new Identity(IdentityType.EMAIL, "a@b");
			voMan.addIdentity(tested);
			Group scope = new Group("/g");
			voMan.addGroup(null, scope.getName());
			Group g = voMan.addGroup(scope, "sub");
			voMan.addGroup(scope, "sub2");
			voMan.addToGroup(tested, g);
			
			conf.doHttpAuthn = true;
			conf.httpPasswd = "";
			conf.httpUser = "a@b";
			conf.doSSLAuthn = false;
			SAMLVOQueryClient client = new SAMLVOQueryClient(addr, conf, conf.getConfiguration());
			//this tests both authentication and SAML query subject (both should succeed) 
			List<Attribute> attrs = client.getAttributes(tested, scope);
			if (attrs.size() != 1)
				fail("Should get 1 attribute");
			Attribute a = attrs.get(0);
			List<String> values = a.getValues();
			assertTrue("Should get two groups", values.size() == 2);
		} catch (Exception e)
		{
			e.printStackTrace();
			fail("User was not authenticated");
		}
		
	}

	public void testMultiQuery()
	{
		try
		{
			Identity tested = new Identity(IdentityType.EMAIL, "a2@b");
			voMan.addIdentity(tested);
			Group scope = new Group("/g2");
			voMan.addGroup(null, scope.getName());
			Group g = voMan.addGroup(scope, "sub");
			voMan.addGroup(scope, "sub2");
			voMan.addToGroup(tested, g);
			
			
			voMan.setAttribute(new Element(null, tested), new Attribute("a1", 
				new ArrayList<String>(), null), false);
			List<String> vals = new ArrayList<String>();
			vals.add("v1");
			vals.add("v2");
			voMan.setAttribute(new Element(scope, tested), new Attribute("a2", 
				vals, null), false);
			voMan.setAttribute(new Element(scope, tested), new Attribute("a3", 
				Collections.singletonList("v3"), null), false);
			
			conf.doHttpAuthn = true;
			conf.httpPasswd = "";
			conf.httpUser = "a2@b";
			conf.doSSLAuthn = false;
			SAMLVOQueryClient client = new SAMLVOQueryClient(addr, conf, conf.getConfiguration());
			
			List<Attribute> toQuery = new ArrayList<Attribute>();
			toQuery.add(new Attribute("a1", Collections.singletonList("@/"), null));
			toQuery.add(new Attribute("a2", Collections.singletonList("v1@" + scope.toString()), null));
			toQuery.add(new Attribute("a2", Collections.singletonList("v2@" + g.toString()), null));
			toQuery.add(new Attribute("a3", Collections.singletonList("vD@" + scope.toString()), null));
			Assertion assertion = client.getAssertionSimpleScoped(tested, toQuery);
			
			if (assertion == null)
				fail("Got no assertion");

			AttributeStatementType []xmlAttrStatements = assertion.getAttributes();
			int f=0;
			for (AttributeStatementType xmlAttrStatement: xmlAttrStatements)
			{
				AttributeType xmlAttrs[] = xmlAttrStatement.getAttributeArray();
				for (AttributeType xmlAttr: xmlAttrs)
				{
					List<Attribute> attr = SAMLXMLBeansMapper.map2APIAttributes(
							xmlAttr);
					for (Attribute a: attr)
					{
						f += a.getValues().size() + 1;
					}
				}
			}
			assertTrue("Got invalid answer " + f, f == 3);
		} catch (Exception e)
		{
			e.printStackTrace();
			fail("User was not authenticated");
		}
		
	}

	
	public void testX509AsDNAttributes()
	{
		try
		{
			FileInputStream fis = new FileInputStream("src/test/resources/certs/pubCert.pem");
			int len = fis.available();
			byte []buffer = new byte[len];
			fis.read(buffer);
			Identity idCert = new Identity(IdentityType.X509CERT,  new String(buffer), null);
			
			voMan.addIdentity(idCert);
			voMan.setAttribute(new Element(null, idCert), 
				new Attribute("attr", null, null), true);

			conf.doHttpAuthn = true;
			conf.httpPasswd = "";
			conf.httpUser = "voadmin@localhost";
			conf.doSSLAuthn = false;
			String dn = new X509IdentityHelper(idCert).getCertificate().
				getSubjectX500Principal().getName();
			Identity dnId = new Identity(IdentityType.DN, dn);
			
			//ask normally for X509 identity attrs
			SAMLVOQueryClient client = new SAMLVOQueryClient(addr, conf, conf.getConfiguration());
			List<Attribute> attrs = client.getAttributes(idCert);
			if (attrs.size() != 1)
				fail("Should get 1 attribute");
			Attribute a = attrs.get(0);
			assertTrue("Should get other attribute", a.getName().equals("attr"));
			
			//now mapping should be applied - ask for DN type identity, while there is
			//only cert with the same subject
			attrs = client.getAttributes(dnId);
			if (attrs.size() != 1)
				fail("Should get 1 attribute");
			a = attrs.get(0);
			assertTrue("Should get other attribute", a.getName().equals("attr"));
			
			//now as dn was added mapping shall not be used anymore
			voMan.addIdentity(dnId);
			attrs = client.getAttributes(dnId);
			if (attrs.size() != 0)
				fail("Should get 0 attributes");
		} catch (Exception e)
		{
			e.printStackTrace();
			fail("User was not authenticated");
		}
		
	}

}
