/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Apr 12, 2008
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.uvos.test;

import java.net.URI;

import eu.unicore.samly2.SAMLConstants;
import eu.unicore.samly2.assertion.Assertion;

import junit.framework.TestCase;
import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.db.InitDB;
import pl.edu.icm.unicore.uvos.server.ComponentContainer;
import pl.edu.icm.unicore.uvos.server.VOServer;
import pl.edu.icm.unicore.uvos.util.Configuration;
import pl.edu.icm.unicore.uvos.wsclient.samlapi.SAMLVOAuthnClientPOST;
import pl.edu.icm.unicore.uvos.wsclient.samlapi.SAMLVOAuthnClientSOAP;

/**
 * Test of SAML authentication protocol with SOAP binding.
 * @author K. Benedyczak
 */
public class SAMLAuthNTest extends TestCase
{
	private static final String UVOS_CONF = "src/test/resources/uvos/uvosServer.conf";

	private static boolean initDone = false;
	protected String addr;
	protected TestSecurityConfig conf;
	
	@Override
	protected void setUp()
	{
		try
		{
			Configuration c = new Configuration("src/test/resources/functionalTest.conf");
			conf = new TestSecurityConfig(c);
			
			addr = conf.getProtocol() + "://" + 
				conf.getServerHost() + ":" + conf.getServerPort();
		} catch (Throwable e)
		{
			System.err.println("Severe error when creating configuration");
			e.printStackTrace();
			fail();
		}

		if (initDone)
			return;
		try
		{
			InitDB.main(new String[]{UVOS_CONF, "-b"});
			ComponentContainer container = new ComponentContainer(UVOS_CONF);
			VOServer serverInstance = container.getComponent(VOServer.class);
			serverInstance.startup(container.getContainer());
		} catch (Throwable e)
		{
			System.err.println("Severe error when starting UVOS Web Server:");
			e.printStackTrace();
			fail();
		}
		initDone = true;
	}
	
	public void test()
	{
		try
		{			
			conf.doHttpAuthn = true;
			conf.httpPasswd = "";
			conf.httpUser = "voadmin@localhost";
			conf.doSSLAuthn = false;
			SAMLVOAuthnClientSOAP client = new SAMLVOAuthnClientSOAP(addr, conf,
					new URI("http://serviceprovider.test.org"), conf.getConfiguration());
			Assertion a1 = client.authenticate(null);
			Identity i1 = SAMLVOAuthnClientPOST.getIdentityFromAssertion(a1);
			if (!i1.getType().getName().equals(SAMLConstants.NFORMAT_PERSISTENT))
			{
				System.err.println("Returned identity type is not PERSISTANT: "
						+ i1);
				fail("Returned identity type is not PERSISTANT");
			}
			Assertion a2 = client.authenticate(SAMLConstants.NFORMAT_EMAIL);
			Identity i2 = SAMLVOAuthnClientPOST.getIdentityFromAssertion(a2);
			if (!i2.getType().getName().equals(SAMLConstants.NFORMAT_EMAIL)
					|| !i2.getValue().equals("voadmin@localhost"))
			{
				System.err.println("Returned identity is wrong: "
						+ i2);
				fail("Returned identity type is wrong");
			}
//			System.out.println("Authenticated as: " + 
//					a1.getXML().xmlText(new XmlOptions().setSavePrettyPrint()));
		} catch (Exception e)
		{
			e.printStackTrace();
			fail("User was not authenticated");
		}
		
	}
}
