/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Apr 12, 2008
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.uvos.test;

import java.io.FileInputStream;
import java.security.KeyStore;
import java.security.PrivateKey;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import eu.unicore.security.etd.DelegationRestrictions;
import eu.unicore.security.etd.ETDApi;
import eu.unicore.security.etd.ETDImpl;
import eu.unicore.security.etd.TrustDelegation;

import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.IdentityType;
import pl.edu.icm.unicore.uvos.api.Permissions;
import pl.edu.icm.unicore.uvos.db.InitDB;
import pl.edu.icm.unicore.uvos.engine.VirtualSecurityContextImpl;
import pl.edu.icm.unicore.uvos.engine.api.UVOSManagementInterface;
import pl.edu.icm.unicore.uvos.server.ComponentContainer;
import pl.edu.icm.unicore.uvos.server.VOServer;
import pl.edu.icm.unicore.uvos.server.idresolver.Authenticator;
import pl.edu.icm.unicore.uvos.util.Configuration;
import pl.edu.icm.unicore.uvos.wsclient.api.UVOSClientFactory;
import pl.edu.icm.unicore.uvos.wsclient.api.WSManagementClient;
import junit.framework.TestCase;

/**
 * TD tests.
 * @author K. Benedyczak
 */
public class TDTest extends TestCase
{
	private static final String UVOS_CONF = "src/test/resources/uvos/uvosServer.conf";
	private static final String CLIENT2_KS = "src/test/resources/certs/dummyClient2.jks";
	private static final String CLIENT2_KS_PASSWD = "the!client";
	private static final String CLIENT2_KS_ALIAS = "mykey";
	
	private static boolean initDone = false;

	private String addr;
	private TestSecurityConfig conf;
	
	
	@Override
	protected void setUp()
	{
		if (initDone)
			return;
		try
		{
			InitDB.main(new String[]{UVOS_CONF, "-b"});
			ComponentContainer container = new ComponentContainer(UVOS_CONF);
			VOServer serverInstance = container.getComponent(VOServer.class);
			serverInstance.startup(container.getContainer());
			UVOSManagementInterface man = container.getComponent(UVOSManagementInterface.class);
			Identity client2 = new Identity(IdentityType.DN, 
					"CN=Example Client 2, OU=Example OU, O=Example O, " +
					"L=Example City, ST=Example Province, C=PL");
			Authenticator.injectFakeContext(new VirtualSecurityContextImpl(Permissions.ALL_P));
			man.addIdentity(client2);
			Authenticator.injectFakeContext(null);
			initDone = true;
		} catch (Throwable e)
		{
			System.err.println("Severe error when starting UVOS Web Server:");
			e.printStackTrace();
			fail();
		}

		try
		{
			conf = null;
			Configuration c = new Configuration("src/test/resources/functionalTest.conf");
			conf = new TestSecurityConfig(c);
		
			addr = conf.getProtocol() + "://" + 
				conf.getServerHost() + ":" + conf.getServerPort();
		} catch (Throwable e)
		{
			System.err.println("Severe error when starting UVOS Web Server:");
			e.printStackTrace();
			fail();
		}
	}
	
	public void test1()
	{
		try
		{
			doWork();
		} catch (Exception e)
		{
			e.printStackTrace();
			fail(e.toString());
		}
	}
	
	/**
	 * ExampleClient2 delegates his trust to ExampleClient which uses this 
	 * delegation to access the service.
	 *
	 */
	private void doWork() throws Exception
	{
		ETDApi etdEngine = new ETDImpl();
		
		List<TrustDelegation> tdChain = new ArrayList<TrustDelegation>();
		Calendar c = Calendar.getInstance();
		c.add(Calendar.HOUR, 1);
		DelegationRestrictions rest = new DelegationRestrictions(new Date(), 
				c.getTime(), 0);
		String receiverDN = "CN=Example Client, OU=Example OU, O=Example O, " +
				"L=Example City, ST=Example Province, C=PL";
		
		KeyStore client2Ks = KeyStore.getInstance("JKS");
		client2Ks.load(new FileInputStream(CLIENT2_KS), 
				CLIENT2_KS_PASSWD.toCharArray());
		PrivateKey privKey = (PrivateKey) client2Ks.getKey(CLIENT2_KS_ALIAS, 
				CLIENT2_KS_PASSWD.toCharArray());
		X509Certificate cert = (X509Certificate) client2Ks.getCertificate(
				CLIENT2_KS_ALIAS);
		TrustDelegation td = etdEngine.generateTD(cert.getSubjectX500Principal().getName(),
				new X509Certificate[] {cert}, privKey, receiverDN, rest);
		tdChain.add(td);
		conf.doSSLAuthn = true;
		UVOSClientFactory fact = new UVOSClientFactory(conf, conf.getConfiguration());
		fact.setTD(tdChain);
		
		WSManagementClient man = fact.getWSManagementClientHL(addr);
		man.getIdentityTypes();
	}
}
