package pl.edu.icm.uvos.test.unit;

import java.util.ArrayList;
import java.util.Date;

import junit.framework.TestCase;

import org.jmock.Expectations;
import org.jmock.Mockery;

import pl.edu.icm.unicore.uvos.api.ApplicationActions;
import pl.edu.icm.unicore.uvos.api.ApplicationStatus;
import pl.edu.icm.unicore.uvos.api.Attribute;
import pl.edu.icm.unicore.uvos.api.Element;
import pl.edu.icm.unicore.uvos.api.Group;
import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.IdentityType;
import pl.edu.icm.unicore.uvos.api.PermissionDesignator;
import pl.edu.icm.unicore.uvos.api.Permissions;
import pl.edu.icm.unicore.uvos.api.VOApplication;
import pl.edu.icm.unicore.uvos.clc.CLCExecutor;
import pl.edu.icm.unicore.uvos.clc.ConnectionManager;
import pl.edu.icm.unicore.uvos.wsclient.api.UVOSAuthZExtInterface;
import pl.edu.icm.unicore.uvos.wsclient.api.UVOSGLUEExtInterface;
import pl.edu.icm.unicore.uvos.wsclient.api.UVOSManagementExtInterface;
import pl.edu.icm.unicore.uvos.wsclient.api.UVOSQueryExtInterface;
import pl.edu.icm.unicore.uvos.wsclient.api.UVOSQueryHistoryExtInterface;
import pl.edu.icm.unicore.uvos.wsclient.api.VOApplicationExtInterface;
import pl.edu.icm.unicore.uvos.api.ModificationEvent;

public class ActionTest extends TestCase {
	Mockery context;
	UVOSManagementExtInterface voMan;
	UVOSAuthZExtInterface voAuthz;
	UVOSQueryExtInterface voQuery;
	UVOSQueryHistoryExtInterface voQueryHistory;
	VOApplicationExtInterface voApps;
	UVOSGLUEExtInterface glueClient;

	CLCExecutor CLClientExec;

	@Override
	protected void setUp() throws Exception {
		context = new Mockery();
		voMan = context.mock(UVOSManagementExtInterface.class);
		voAuthz = context.mock(UVOSAuthZExtInterface.class);
		voQuery = context.mock(UVOSQueryExtInterface.class);
		voQueryHistory = context.mock(UVOSQueryHistoryExtInterface.class);
		voApps = context.mock(VOApplicationExtInterface.class);
		glueClient = context.mock(UVOSGLUEExtInterface.class);
		
		ConnectionManager connectionManager = new ConnectionManager(voMan,
				voAuthz, voQuery, voQueryHistory, voApps, glueClient, null);
		CLClientExec = new CLCExecutor(connectionManager);
		CLClientExec.registerAllActions();
	}

	public void testGroupActions() {
		final Group ParentG = new Group("/");
		final Group GroupA = new Group("A");
		final Group CopyGroupA = new Group("COPYA");
		final Identity id = new Identity(IdentityType.EMAIL, "ann@example.com",
				null);

		try {
			context.checking(new Expectations() {
				{
					oneOf(voMan).addGroup(with(equal(ParentG)),
							with(equal("A")));
					will(returnValue(GroupA));
					oneOf(voMan).copyGroup(with(equal(GroupA)),
							with(equal(ParentG)), with(equal("COPYA")),
							with(equal(false)));
					oneOf(voMan).removeGroup(with(equal(GroupA)),
							with(equal(true)));
					oneOf(voMan).removeGroup(with(equal(CopyGroupA)),
							with(equal(true)));

					oneOf(voMan).removeFromGroup(with(equal(id)),
							with(equal(GroupA)));

					oneOf(voMan).addToGroup(with(equal(id)),
							with(equal(GroupA)));

					oneOf(voQuery).getAllGroups(with(equal(id)),
							with(equal(false)));
				}
			});

		} catch (Exception e) {
			fail();
			e.printStackTrace();
		}

		try {

			CLClientExec.invokeInteractiveCommand("addGroup / A");
			CLClientExec.invokeInteractiveCommand("copyGroup /A / false COPYA");
			CLClientExec.invokeInteractiveCommand("removeGroup /A true");
			CLClientExec.invokeInteractiveCommand("removeGroup /COPYA true");
			CLClientExec.invokeInteractiveCommand("getAllGroups email ann@example.com");
			CLClientExec.invokeInteractiveCommand("removeFromGroup email ann@example.com /A");
			CLClientExec.invokeInteractiveCommand("addToGroup email ann@example.com /A");

		} catch (Exception e1) {
			fail("Group action fail");
			e1.printStackTrace();
		}

		context.assertIsSatisfied();
	}

	public void testIdentytiActions() {

		final Identity id = new Identity(IdentityType.EMAIL,
				"test@example.com", "Johnny");
		final Identity rid = new Identity(IdentityType.EMAIL,
				"test@example.com", null);

		try {
			context.checking(new Expectations() {
				{
					oneOf(voMan).addIdentity(with(equal(id)));
					oneOf(voMan).removeIdentity(with(equal(rid)));
					oneOf(voMan)
							.addIdentity(with(equal(rid)), with(equal(rid)));

					oneOf(voAuthz).getMyIdentities();
					oneOf(voQuery).getAllIdentities();
				}
			});

		} catch (Exception e) {
			fail();
			e.printStackTrace();
		}

		try {

			CLClientExec.invokeInteractiveCommand("addIdentity email test@example.com Johnny");
			CLClientExec.invokeInteractiveCommand("removeIdentity email test@example.com");

			CLClientExec.invokeInteractiveCommand("getMyIds");

			CLClientExec.invokeInteractiveCommand("getAllIdentities");
			CLClientExec.invokeInteractiveCommand("addEquivalentIdentity email test@example.com email test@example.com");

		} catch (Exception e1) {
			fail("Identity action fail");
			e1.printStackTrace();
		}

		context.assertIsSatisfied();

	}

	public void testNotificationAction() {

		try {
			context.checking(new Expectations() {
				{
					oneOf(voMan).addNotification(with(equal("addGroup")),
							with(equal("receiver@example.com")),
							with(equal("/Math-VO")));
					oneOf(voMan).removeNotification(with(equal(3)));
					oneOf(voMan).getNotifications(with(equal("addGroup")),
							with(equal("/Math-VO")));

				}
			});

		} catch (Exception e) {
			fail();
			e.printStackTrace();
		}

		try {

			CLClientExec.invokeInteractiveCommand("addNotification addGroup receiver@example.com /Math-VO");
			CLClientExec.invokeInteractiveCommand("removeNotification 3");
			CLClientExec.invokeInteractiveCommand("getNotifications addGroup /Math-VO");

		} catch (Exception e1) {
			fail("Notification action fail");
			e1.printStackTrace();
		}

		context.assertIsSatisfied();

	}

	public void testAuthAction() {
		final Group g = new Group("/some/group");
		final Permissions p = new Permissions(Permissions.FULL_READ);

		try {
			context.checking(new Expectations() {
				{
					exactly(3).of(voAuthz).modifyPermissions(with(equal(g)),
							with(aNonNull(PermissionDesignator.class)),
							with(equal(p)));
					exactly(3).of(voAuthz).modifyPermissions(with(equal(g)),
							with(aNonNull(PermissionDesignator.class)),
							(Permissions) with(equal(null)));
					oneOf(voAuthz).getGroupAuthZ((Group) with(equal(null)),
							with(equal(false)));
					exactly(2).of(voAuthz).getGroupAuthZ(with(equal(g)),
							with(equal(false)));
					exactly(2).of(voAuthz).getGroupAuthZ(with(equal(g)),
							with(equal(true)));

				}
			});

		} catch (Exception e) {
			fail();
			e.printStackTrace();
		}
		try {

			CLClientExec.invokeInteractiveCommand("setAuthz /some/group m -f--");
			CLClientExec.invokeInteractiveCommand("setAuthz /some/group o -f--");
			CLClientExec.invokeInteractiveCommand("setAuthz /some/group a -f-- null");
			CLClientExec.invokeInteractiveCommand("removeAuthz /some/group m");
			CLClientExec.invokeInteractiveCommand("removeAuthz /some/group o");
			CLClientExec.invokeInteractiveCommand("removeAuthz /some/group a null");
			CLClientExec.invokeInteractiveCommand("getAuthz global");
			CLClientExec.invokeInteractiveCommand("getAuthz group /some/group o");
			CLClientExec.invokeInteractiveCommand("getAuthz group /some/group a");

		} catch (Exception e1) {
			fail("Auth action fail");
			e1.printStackTrace();
		}

		context.assertIsSatisfied();
	}

	public void testAttributeAction() {
		try {
			context.checking(new Expectations() {
				{
					oneOf(voMan).enableAttribute(with(aNonNull(Element.class)),
							with(aNonNull(String.class)),
							with(aNonNull(String.class)));
					oneOf(voMan).disableAttribute(
							with(aNonNull(Element.class)),
							with(aNonNull(String.class)),
							with(aNonNull(String.class)));
					oneOf(voQuery).getAttributes(with(aNonNull(Element.class)),
							with(aNonNull(String.class)), with(equal(true)),
							with(aNonNull(Boolean.class)),
							with(aNonNull(Boolean.class)));
					oneOf(voQuery).getAttributes(with(aNonNull(Element.class)),
							with(aNonNull(String.class)), with(equal(false)),
							with(aNonNull(Boolean.class)),
							with(aNonNull(Boolean.class)));

					oneOf(voQuery).getAttributes(with(aNonNull(Element.class)),
							(String) with(equal(null)), with(equal(true)),
							with(aNonNull(Boolean.class)),
							with(aNonNull(Boolean.class)));
					oneOf(voQuery).getAttributes(with(aNonNull(Element.class)),
							(String) with(equal(null)), with(equal(false)),
							with(aNonNull(Boolean.class)),
							with(aNonNull(Boolean.class)));
					oneOf(voMan).getDisabledAttributes(
							with(aNonNull(Element.class)));
					oneOf(voMan).removeAttribute(with(aNonNull(Element.class)),
							with(aNonNull(String.class)));

					oneOf(voMan).setAttribute(with(aNonNull(Element.class)),
							with(aNonNull(Attribute.class)),
							with(aNonNull(Boolean.class)));
				}
			});

		} catch (Exception e) {
			fail();
			e.printStackTrace();
		}
		try {

			CLClientExec.invokeInteractiveCommand("enableAttribute ig email ann@example.com /group urn:unicore:attrType:user:proffession scientist");
			CLClientExec.invokeInteractiveCommand("disableAttribute ig email ann@example.com /group urn:unicore:attrType:user:proffession scientist");
			CLClientExec.invokeInteractiveCommand("getAttribute ig email ann@example.com /group urn:unicore:attrType:user:proffession");
			CLClientExec.invokeInteractiveCommand("getAttributes ig email ann@example.com /group");
			CLClientExec.invokeInteractiveCommand("getDisabledAttributes ig email ann@example.com /group");
			CLClientExec.invokeInteractiveCommand("removeAttribute ig email ann@example.com /group urn:unicore:attrType:user:proffession");
			CLClientExec.invokeInteractiveCommand(" setAttribute ig email ann@example.com /group true urn:unicore:attrType:user:proffession scientist");

		} catch (Exception e1) {
			fail("Attribute action fail");
			e1.printStackTrace();
		}

		context.assertIsSatisfied();

	}

	public void testApplicationAndApplicationFormAction() {

		try {
			context.checking(new Expectations() {
				{
					oneOf(voApps).getApplications((Integer) with(equal(null)),
							(ApplicationStatus) with(equal(null)));
					oneOf(voApps).getApplicationForms();
					oneOf(voApps).getApplications(
							with(aNonNull(Integer.class)),
							with(aNonNull(ApplicationStatus.class)));
					oneOf(voApps).processApplication(with(equal(1)),
							with(equal(ApplicationActions.REJECT)),
							with(aNonNull(String.class)),
							with(aNonNull(Boolean.class)),
							(VOApplication) with(equal(null)));
					oneOf(voApps).removeApplicationForm(with(equal(1)));
				}
			});

		} catch (Exception e) {
			fail();
			e.printStackTrace();
		}
		try {

			CLClientExec.invokeInteractiveCommand("getApplication email test@test.com");
			CLClientExec.invokeInteractiveCommand("getApplicationForms");
			CLClientExec.invokeInteractiveCommand("getApplications 1 "
					+ ApplicationStatus.ACCEPTED);
			CLClientExec.invokeInteractiveCommand("processApplication 1 REJECT true notes");
			CLClientExec.invokeInteractiveCommand("removeApplicationForm 1");
		} catch (Exception e1) {
			fail("Application action fail");
			e1.printStackTrace();
		}

		context.assertIsSatisfied();

	}

	public void testOtherAction(){
		try {
			context.checking(new Expectations() {
				{
				oneOf(voAuthz).modifyAuthenticationData(with(aNonNull(Identity.class)), with(aNonNull(String.class)));
				oneOf(voMan).purgeHistoricalData(with(aNonNull(Date.class)));
				oneOf(voAuthz).checkPermissions(with(aNonNull(Group.class)), with(aNonNull(Identity.class)));
				oneOf(voQueryHistory).getHistoryEvents(with(aNonNull(Date.class)), with(aNonNull(Date.class)), with(aNonNull(Integer.class)),  with(aNonNull(Integer.class)), with(aNonNull(Boolean.class)));will(returnValue(new  ArrayList<ModificationEvent>()));
				oneOf(voQuery).areEquivalent(with(aNonNull(Identity.class)),with(aNonNull(Identity.class)));
				}
			});

		} catch (Exception e) {
			fail();
			e.printStackTrace();
		}
		try {
			
			CLClientExec.invokeInteractiveCommand("changePasswd email ann@example.com SecreT");
			CLClientExec.invokeInteractiveCommand("purgeHistory 2007-03-28 21:49:00");
			CLClientExec.invokeInteractiveCommand("getPerms group /some/group email ann@example.com");
			CLClientExec.invokeInteractiveCommand("getEvents from 2007-03-28 21:49:00 to 2008-03-28 21:49:00");
			CLClientExec.invokeInteractiveCommand("areEquivalent email ann@example.com email ann@example.org");
		
		} catch (Exception e1) {
			fail("Action fail");
			e1.printStackTrace();
		}

		context.assertIsSatisfied();

		try {
			CLClientExec.invokeInteractiveCommand("setTime 2007-01-01 23:56:00");
		} catch (Exception e1) {
			fail("Action setTime fail");
			e1.printStackTrace();
		}
		
	}
	
	
}
