package pl.edu.icm.uvos.test.unit;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;

import junit.framework.TestCase;

import org.jmock.Expectations;
import org.jmock.Mockery;

import pl.edu.icm.unicore.uvos.api.ApplicationActions;
import pl.edu.icm.unicore.uvos.api.Attribute;
import pl.edu.icm.unicore.uvos.api.Element;
import pl.edu.icm.unicore.uvos.api.Group;
import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.IdentityType;
import pl.edu.icm.unicore.uvos.api.PermissionDesignator;
import pl.edu.icm.unicore.uvos.api.Permissions;
import pl.edu.icm.unicore.uvos.api.VOApplication;
import pl.edu.icm.unicore.uvos.api.VOApplicationForm;
import pl.edu.icm.unicore.uvos.api.PermissionDesignator.SpecialDesignator;
import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.UVOSFault;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.WSApplicationsInterface;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.WSAuthZInterface;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.WSQueryHistoryInterface;
import pl.edu.icm.unicore.uvos.wsclient.api.WSApplicationClient;
import pl.edu.icm.unicore.uvos.wsclient.api.WSAuthZClient;
import pl.edu.icm.unicore.uvos.wsclient.api.WSQueryHistoryClient;
import xmlbeans.pl.edu.icm.uvos.internal.ElementDocument;
import xmlbeans.pl.edu.icm.uvos.internal.GroupDocument;
import xmlbeans.pl.edu.icm.uvos.internal.IdentityDocument;
import xmlbeans.pl.edu.icm.uvos.internal.PermissionsDesignatorDocument;
import xmlbeans.pl.edu.icm.uvos.internal.PermissionsDocument;
import xmlbeans.pl.edu.icm.uvos.internal.app.ProcessVOApplicationDocument;
import xmlbeans.pl.edu.icm.uvos.internal.app.VOApplicationDocument;
import xmlbeans.pl.edu.icm.uvos.internal.app.VOApplicationFormConstraintsType;
import xmlbeans.pl.edu.icm.uvos.internal.app.VOApplicationFormDocument;
import xmlbeans.pl.edu.icm.uvos.internal.app.VOApplicationRequestsDocument;

public class ApiImplTest extends TestCase {

	public void testWSApplicationClient() {
		Mockery context = new Mockery();
		final WSApplicationsInterface proxy = context
				.mock(WSApplicationsInterface.class);
		WSApplicationClient cl = new WSApplicationClient(proxy);

		try {
			context.checking(new Expectations() {
				{
					oneOf(proxy).getApplicationForms();
					oneOf(proxy).removeApplicationForm(with(equal(1)));
					oneOf(proxy).updateApplicationForm(
							with(aNonNull(VOApplicationFormDocument.class)),
							with(equal(false)));
					will(returnValue(null));
					oneOf(proxy).getApplications(with(equal(1)),
							(String) with(equal(null)));
					oneOf(proxy).submitApplication(
							with(aNonNull(VOApplicationDocument.class)));
					oneOf(proxy).processApplication(
							with(aNonNull(ProcessVOApplicationDocument.class)));
					oneOf(proxy).csrProcessedNotification(
							with(aNonNull(String.class)), with(equal(false)),
							with(aNonNull(String.class)), with(equal(false)));
				}
			});
		} catch (UVOSFault e) {
			e.printStackTrace();
			fail(e.getMessage());
		}

		try {
			cl.getApplicationForms();
			cl.removeApplicationForm(1);
			Group g = new Group("/");
			VOApplicationFormConstraintsType con = VOApplicationFormConstraintsType.Factory
					.newInstance();
			VOApplicationForm f1 = new VOApplicationForm("form1", "form", 1,
					"agree", g, con.toString());
			cl.updateApplicationForm(f1, false);
			cl.getApplications(1, null);
			Identity id = new Identity(IdentityType.EMAIL, "test1@test.com");

			VOApplication a = new VOApplication(1, 1, id, false,
					"test@test.com", VOApplicationRequestsDocument.Factory
							.newInstance().toString(), id);
			cl.submitApplication(a);

			cl
					.processApplication(1, ApplicationActions.ACCEPT, "note",
							true, a);
			cl.csrProcessedNotification("csr", false, "cert", false);

		} catch (UVOSException e) {
			e.printStackTrace();
			fail(e.getMessage());
		}

		context.assertIsSatisfied();

	}

	public void testWSAuthZClient() {
		Mockery context = new Mockery();
		final WSAuthZInterface proxy = context.mock(WSAuthZInterface.class);
		WSAuthZClient cl = new WSAuthZClient(proxy);
		Identity id = new Identity(IdentityType.EMAIL, "test1@test.com");
		Group g = new Group("/A");

		try {
			context.checking(new Expectations() {
				{

					oneOf(proxy).checkPermissions(
							with(aNonNull(GroupDocument.class)),
							with(aNonNull(IdentityDocument.class)));
					oneOf(proxy).getGroupAuthZ(
							with(aNonNull(GroupDocument.class)),
							with(equal(false)));
					oneOf(proxy).modifyAuthenticationData(
							with(aNonNull(IdentityDocument.class)),
							with(aNonNull(String.class)));
					oneOf(proxy)
							.modifyPermissions(
									with(aNonNull(GroupDocument.class)),
									with(aNonNull(PermissionsDesignatorDocument.class)),
									with(aNonNull(PermissionsDocument.class)));
					oneOf(proxy).getMyIdentities();
				}
			});
		} catch (UVOSFault e) {
			e.printStackTrace();
			fail(e.getMessage());
		}

		try {
			cl.checkPermissions(g, id);
			cl.getGroupAuthZ(g, false);
			cl.modifyAuthenticationData(id, 1);
			SpecialDesignator spec = SpecialDesignator.MEMBER;
			PermissionDesignator d = new PermissionDesignator(spec);
			cl.modifyPermissions(g, d, new Permissions(2));
			cl.getMyIdentities();

		} catch (UVOSException e) {
			e.printStackTrace();
			fail(e.getMessage());
		}

		context.assertIsSatisfied();

	}

	public void testWSQueryHistoryClient() {
		Mockery context = new Mockery();
		final WSQueryHistoryInterface proxy = context
				.mock(WSQueryHistoryInterface.class);
		WSQueryHistoryClient cl = new WSQueryHistoryClient(proxy);

		try {
			context.checking(new Expectations() {
				{

					oneOf(proxy).areEquivalent(
							with(aNonNull(IdentityDocument.class)),
							with(aNonNull(IdentityDocument.class)),
							with(aNonNull(Date.class)));
					oneOf(proxy).getAttributes(
							with(aNonNull(ElementDocument.class)),
							with(aNonNull(String.class)), with(equal(false)),
							with(equal(false)), with(equal(false)),
							with(aNonNull(Date.class)));
					oneOf(proxy).getAllEquivalents(
							with(aNonNull(IdentityDocument.class)),
							with(aNonNull(Date.class)));
					oneOf(proxy).getAllGroups(
							with(aNonNull(IdentityDocument.class)),
							with(aNonNull(Boolean.class)),
							with(aNonNull(Date.class)));
					oneOf(proxy).getAllIdentities(with(aNonNull(Date.class)));
					oneOf(proxy).getHistoryEvents(with(aNonNull(Date.class)),
							with(aNonNull(Date.class)),
							with(aNonNull(Integer.class)),
							with(aNonNull(Integer.class)), with(equal(false)));
				}
			});
		} catch (UVOSFault e) {
			e.printStackTrace();
			fail(e.getMessage());
		}

		try {
			Identity id = new Identity(IdentityType.EMAIL, "test1@test.com");
			Group g = new Group("/A");
			cl.areEquivalent(id, id, Calendar.getInstance().getTime());
			Element e = new Element(g, id);
			Attribute a = new Attribute("A", new ArrayList<String>(), null);
			Date d = Calendar.getInstance().getTime();
			cl.getAttributes(e, a.toString(), false, false, false, d);
			cl.getAllEquivalents(id, d);
			cl.getAllGroups(id, true, d);
			cl.getAllIdentities(d);
			cl.getHistoryEvents(d, d, 1, 1, false);

		} catch (UVOSException e) {
			e.printStackTrace();
			fail(e.getMessage());
		}

		context.assertIsSatisfied();

	}

}
